from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 账户与交易
class _CMAccountTradeEndpoints():
    set_positionSide_dual = ['https://dapi.binance.com/','POST', '/dapi/v1/positionSide/dual', True]  # 更改持仓模式(TRADE)
    get_positionSide_dual = ['https://dapi.binance.com/','GET', '/dapi/v1/positionSide/dual', True]  # 查询持仓模式(USER_DATA)
    set_order = ['https://dapi.binance.com/','POST', '/dapi/v1/order', True]  # 下单 (TRADE)
    set_order_test = ['https://dapi.binance.com/','POST', '/dapi/v1/order/test', True]  # 测试下单接口 (TRADE)
    alter_order = ['https://dapi.binance.com/','PUT', '/dapi/v1/order', True]  # 修改订单 (TRADE)
    set_batchOrders = ['https://dapi.binance.com/','POST', '/dapi/v1/batchOrders', True]  # 批量下单 (TRADE)
    alter_batchOrders = ['https://dapi.binance.com/','PUT', '/dapi/v1/batchOrders', True]  # 批量修改订单 (TRADE)
    get_orderAmendment = ['https://dapi.binance.com/','GET', '/dapi/v1/orderAmendment', True]  # 查询订单修改历史 (USER_DATA)
    get_order = ['https://dapi.binance.com/','GET', '/dapi/v1/order', True]  # 查询订单 (USER_DATA)
    cancel_order = ['https://dapi.binance.com/','DELETE', '/dapi/v1/order', True]  # 撤销订单 (TRADE)
    cancel_allOpenOrders = ['https://dapi.binance.com/','DELETE', '/dapi/v1/allOpenOrders', True]  # 撤销全部订单 (TRADE)
    cancel_batchOrders = ['https://dapi.binance.com/','DELETE', '/dapi/v1/batchOrders', True]  # 批量撤销订单 (TRADE)
    set_countdownCancelAll = ['https://dapi.binance.com/','POST', '/dapi/v1/countdownCancelAll', True]  # 倒计时撤销所有订单 (TRADE)
    get_openOrder = ['https://dapi.binance.com/','GET', '/dapi/v1/openOrder', True]  # 查询当前挂单 (USER_DATA)
    get_openOrders = ['https://dapi.binance.com/','GET', '/dapi/v1/openOrders', True]  # 查看当前全部挂单 (USER_DATA)
    get_allOrders = ['https://dapi.binance.com/','GET', '/dapi/v1/allOrders', True]  # 查询所有订单(包括历史订单) (USER_DATA)
    get_balance = ['https://dapi.binance.com/','GET', '/dapi/v1/balance', True]  # 账户余额 (USER_DATA)
    get_account = ['https://dapi.binance.com/','GET', '/dapi/v1/account', True]  # 账户信息 (USER_DATA)
    set_leverage = ['https://dapi.binance.com/','POST', '/dapi/v1/leverage', True]  # 调整开仓杠杆 (TRADE)
    set_marginType = ['https://dapi.binance.com/','POST', '/dapi/v1/marginType', True]  # 变换逐全仓模式 (TRADE)
    set_positionMargin = ['https://dapi.binance.com/','POST', '/dapi/v1/positionMargin', True]  # 调整逐仓保证金 (TRADE)
    get_positionMargin_history = ['https://dapi.binance.com/','GET', '/dapi/v1/positionMargin/history', True]  # 逐仓保证金变动历史 (TRADE)
    get_positionRisk = ['https://dapi.binance.com/','GET', '/dapi/v1/positionRisk', True]  # 用户持仓风险
    get_userTrades = ['https://dapi.binance.com/','GET', '/dapi/v1/userTrades', True]  # 账户成交历史 (USER_DATA)
    get_income = ['https://dapi.binance.com/','GET', '/dapi/v1/income', True]  # 获取账户损益资金流水(USER_DATA)
    get_leverageBracket = ['https://dapi.binance.com/','GET', '/dapi/v2/leverageBracket', False]  # 交易对杠杆分层标准 (USER_DATA)
    get_forceOrders = ['https://dapi.binance.com/','GET', '/dapi/v1/forceOrders', False]  # 用户强平单历史 (USER_DATA)
    get_adlQuantile = ['https://dapi.binance.com/','GET', '/dapi/v1/adlQuantile', False]  # 持仓ADL队列估算 (USER_DATA)
    get_commissionRate = ['https://dapi.binance.com/','GET', '/dapi/v1/commissionRate', True]  # 用户手续费率 (USER_DATA)


class CMAccountTrade(Client):
    endpoints = _CMAccountTradeEndpoints

    # 更改持仓模式(TRADE)
    def set_positionSide_dual(self, dualSidePosition: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade

        Name            	Type	Mandatory	Description
        dualSidePosition	str 	YES      	"true": 双向持仓模式；"false": 单向持仓模式
        recvWindow      	int 	NO
        '''
        return self.send_request(*self.endpoints.set_positionSide_dual, **to_local(locals()))

    # 查询持仓模式(USER_DATA)
    def get_positionSide_dual(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_positionSide_dual, **to_local(locals()))

    # 下单 (TRADE)
    def set_order(self, symbol: str = '', side: str = '', positionSide: str = '', type: str = '', reduceOnly: str = '',
                  quantity: Union[float, int] = '', price: Union[float, int] = '', newClientOrderId: str = '',
                  stopPrice: Union[float, int] = '', closePosition: str = '', activationPrice: Union[float, int] = '',
                  callbackRate: Union[float, int] = '', timeInForce: str = '', workingType: str = '',
                  priceProtect: str = '', newOrderRespType: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-2

        Name            	Type            	Mandatory	Description
        symbol          	str             	YES      	交易对
        side            	str             	YES
        positionSide    	str             	NO
        type            	str             	YES
        reduceOnly      	str             	NO
        quantity        	Union[float,int]	NO
        price           	Union[float,int]	NO       	委托价格
        newClientOrderId	str             	NO
        stopPrice       	Union[float,int]	NO
        closePosition   	str             	NO
        activationPrice 	Union[float,int]	NO
        callbackRate    	Union[float,int]	NO
        timeInForce     	str             	NO       	有效方法
        workingType     	str             	NO
        priceProtect    	str             	NO
        newOrderRespType	str             	NO       	"ACK", "RESULT", 默认 "ACK"
        recvWindow      	int             	NO
        '''
        return self.send_request(*self.endpoints.set_order, **to_local(locals()))

    # 测试下单接口 (TRADE)
    def set_order_test(self, symbol: str = '', side: str = '', positionSide: str = '', type: str = '', reduceOnly: str = '',
                  quantity: Union[float, int] = '', price: Union[float, int] = '', newClientOrderId: str = '',
                  stopPrice: Union[float, int] = '', closePosition: str = '', activationPrice: Union[float, int] = '',
                  callbackRate: Union[float, int] = '', timeInForce: str = '', workingType: str = '',
                  priceProtect: str = '', newOrderRespType: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-3

        Name            	Type            	Mandatory	Description
        symbol          	str             	YES      	交易对
        side            	str             	YES
        positionSide    	str             	NO
        type            	str             	YES
        reduceOnly      	str             	NO
        quantity        	Union[float,int]	NO
        price           	Union[float,int]	NO       	委托价格
        newClientOrderId	str             	NO
        stopPrice       	Union[float,int]	NO
        closePosition   	str             	NO
        activationPrice 	Union[float,int]	NO
        callbackRate    	Union[float,int]	NO
        timeInForce     	str             	NO       	有效方法
        workingType     	str             	NO
        priceProtect    	str             	NO
        newOrderRespType	str             	NO       	"ACK", "RESULT", 默认 "ACK"
        recvWindow      	int             	NO
        '''
        return self.send_request(*self.endpoints.set_order_test, **to_local(locals()))

    # 修改订单 (TRADE)
    def alter_order(self, orderId: int = '', origClientOrderId: str = '', symbol: str = '', side: str = '',
                    quantity: Union[float, int] = '', price: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-4

        Name             	Type            	Mandatory	Description
        orderId          	int             	NO       	系统订单号
        origClientOrderId	str             	NO       	用户自定义的订单号
        symbol           	str             	YES      	交易对
        side             	str             	YES
        quantity         	Union[float,int]	NO
        price            	Union[float,int]	NO       	委托价格
        recvWindow       	int             	NO
        '''
        return self.send_request(*self.endpoints.alter_order, **to_local(locals()))

    # 批量下单 (TRADE)
    def set_batchOrders(self, batchOrders: list = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-5

        Name            	Type            	Mandatory	Description
        batchOrders     	list            	YES      	订单列表,最多支持5个订单
        recvWindow      	int             	NO
        '''
        return self.send_request(*self.endpoints.set_batchOrders, **to_local(locals()))

    # 批量修改订单 (TRADE)
    def alter_batchOrders(self, batchOrders: list = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-6

        Name             	Type            	Mandatory	Description
        batchOrders      	list            	YES      	订单列表,最多支持5个订单
        recvWindow       	int             	NO
        '''
        return self.send_request(*self.endpoints.alter_batchOrders, **to_local(locals()))

    # 查询订单修改历史 (USER_DATA)
    def get_orderAmendment(self, symbol: str = '', orderId: int = '', origClientOrderId: str = '', startTime: int = '',
                           endTime: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-2

        Name             	Type	Mandatory	Description
        symbol           	str 	YES      	交易对
        orderId          	int 	NO       	系统订单号
        origClientOrderId	str 	NO       	用户自定义的订单号
        startTime        	int 	NO       	起始时间
        endTime          	int 	NO       	结束时间
        limit            	int 	NO       	返回的结果集数量 默认值:50 最大值:100
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.get_orderAmendment, **to_local(locals()))

    # 查询订单 (USER_DATA)
    def get_order(self, symbol: str = '', orderId: int = '', origClientOrderId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-3

        Name             	Type	Mandatory	Description
        symbol           	str 	YES      	交易对
        orderId          	int 	NO       	系统订单号
        origClientOrderId	str 	NO       	用户自定义的订单号
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.get_order, **to_local(locals()))

    # 撤销订单 (TRADE)
    def cancel_order(self, symbol: str = '', orderId: int = '', origClientOrderId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-7

        Name             	Type	Mandatory	Description
        symbol           	str 	YES      	交易对
        orderId          	int 	NO       	系统订单号
        origClientOrderId	str 	NO       	用户自定义的订单号
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_order, **to_local(locals()))

    # 撤销全部订单 (TRADE)
    def cancel_allOpenOrders(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-8

        Name      	Type	Mandatory	Description
        symbol    	str 	YES      	交易对
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_allOpenOrders, **to_local(locals()))

    # 批量撤销订单 (TRADE)
    def cancel_batchOrders(self, symbol: str = '', orderIdList: list = '',
                           origClientOrderIdList: list = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-9

        Name                 	Type        	Mandatory	Description
        symbol               	str         	YES      	交易对
        orderIdList          	LIST<LONG>  	NO
        origClientOrderIdList	LIST<STRING>	NO
        recvWindow           	int         	NO
        '''
        return self.send_request(*self.endpoints.cancel_batchOrders, **to_local(locals()))

    # 倒计时撤销所有订单 (TRADE)
    def set_countdownCancelAll(self, symbol: str = '', countdownTime: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-10

        Name         	Type	Mandatory	Description
        symbol       	str 	YES
        countdownTime	int 	YES      	倒计时。 1000 表示 1 秒； 0 表示取消倒计时撤单功能。
        recvWindow   	int 	NO
        '''
        return self.send_request(*self.endpoints.set_countdownCancelAll, **to_local(locals()))

    # 查询当前挂单 (USER_DATA)
    def get_openOrder(self, symbol: str = '', orderId: int = '', origClientOrderId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-4

        Name             	Type	Mandatory	Description
        symbol           	str 	YES      	交易对
        orderId          	int 	NO       	系统订单号
        origClientOrderId	str 	NO       	用户自定义的订单号
        recvWindow       	int 	NO
        '''
        return self.send_request(*self.endpoints.get_openOrder, **to_local(locals()))

    # 查看当前全部挂单 (USER_DATA)
    def get_openOrders(self, symbol: str = '', pair: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-5

        Name      	Type	Mandatory	Description
        symbol    	str 	NO       	交易对
        pair      	str 	NO       	标的交易对
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_openOrders, **to_local(locals()))

    # 查询所有订单(包括历史订单) (USER_DATA)
    def get_allOrders(self, symbol: str = '', pair: str = '', orderId: int = '', startTime: int = '', endTime: int = '',
                      limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-6

        Name      	Type	Mandatory	Description
        symbol    	str 	NO       	交易对
        pair      	str 	NO       	标的交易对
        orderId   	int 	NO       	只返回此orderID及之后的订单,缺省返回最近的订单, 仅支持配合symbol使用
        startTime 	int 	NO       	起始时间
        endTime   	int 	NO       	结束时间
        limit     	int 	NO       	返回的结果集数量 默认值:50 最大值:100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_allOrders, **to_local(locals()))

    # 账户余额 (USER_DATA)
    def get_balance(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-7

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_balance, **to_local(locals()))

    # 账户信息 (USER_DATA)
    def get_account(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-8

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_account, **to_local(locals()))

    # 调整开仓杠杆 (TRADE)
    def set_leverage(self, symbol: str = '', leverage: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-11

        Name      	Type	Mandatory	Description
        symbol    	str 	YES      	交易对
        leverage  	int 	YES      	目标杠杆倍数
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_leverage, **to_local(locals()))

    # 变换逐全仓模式 (TRADE)
    def set_marginType(self, symbol: str = '', marginType: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-12

        Name      	Type	Mandatory	Description
        symbol    	str 	YES      	交易对
        marginType	str 	YES      	保证金模式 ISOLATED(逐仓), CROSSED(全仓)
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_marginType, **to_local(locals()))

    # 调整逐仓保证金 (TRADE)
    def set_positionMargin(self, symbol: str = '', positionSide: str = '', amount: Union[float, int] = '',
                           type: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-13

        Name        	Type            	Mandatory	Description
        symbol      	str             	YES      	交易对
        positionSide	str             	NO
        amount      	Union[float,int]	YES      	保证金资金
        type        	int             	YES      	调整方向 1: 增加逐仓保证金,2: 减少逐仓保证金
        recvWindow  	int             	NO
        '''
        return self.send_request(*self.endpoints.set_positionMargin, **to_local(locals()))

    # 逐仓保证金变动历史 (TRADE)
    def get_positionMargin_history(self, symbol: str = '', type: int = '', startTime: int = '', endTime: int = '',
                                   limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#trade-14

        Name      	Type	Mandatory	Description
        symbol    	str 	YES      	交易对
        type      	int 	NO       	调整方向 1: 增加逐仓保证金,2: 减少逐仓保证金
        startTime 	int 	NO       	起始时间
        endTime   	int 	NO       	结束时间
        limit     	int 	NO       	返回的结果集数量 默认值: 50
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_positionMargin_history, **to_local(locals()))

    # 用户持仓风险
    def get_positionRisk(self, marginAsset: str = '', pair: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#479ce129f6

        Name       	Type	Mandatory	Description
        marginAsset	str 	NO
        pair       	str 	NO
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_positionRisk, **to_local(locals()))

    # 账户成交历史 (USER_DATA)
    def get_userTrades(self, symbol: str = '', pair: str = '', startTime: int = '', endTime: int = '', fromId: int = '',
                       limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-9

        Name      	Type	Mandatory	Description
        symbol    	str 	NO       	交易对
        pair      	str 	NO       	标的交易对
        startTime 	int 	NO       	起始时间
        endTime   	int 	NO       	结束时间
        fromId    	int 	NO       	返回该fromId及之后的成交,缺省返回最近的成交,仅支持配合symbol使用
        limit     	int 	NO       	返回的结果集数量 默认值:50 最大值:1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_userTrades, **to_local(locals()))

    # 获取账户损益资金流水(USER_DATA)
    def get_income(self, symbol: str = '', incomeType: str = '', startTime: int = '', endTime: int = '',
                   limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-10

        Name      	Type	Mandatory	Description
        symbol    	str 	NO       	交易对
        incomeType	str 	NO       	收益类型 "TRANSFER","WELCOME_BONUS", "FUNDING_FEE", "REALIZED_PNL", "COMMISSION", "INSURANCE_CLEAR", "DELIVERED_SETTELMENT"
        startTime 	int 	NO       	起始时间
        endTime   	int 	NO       	结束时间
        limit     	int 	NO       	返回的结果集数量 默认值:100 最大值:1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_income, **to_local(locals()))

    # 标的交易对默认杠杆分层标准 (USER_DATA)
    def get_leverageBracket(self, pair: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-11

        Name      	Type	Mandatory	Description
        pair      	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_leverageBracket, **to_local(locals()))

    # 交易对杠杆分层标准 (USER_DATA)
    def get_leverageBracket(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-12

        Name      	Type	Mandatory	Description
        symbol    	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_leverageBracket, **to_local(locals()))

    # 用户强平单历史 (USER_DATA)
    def get_forceOrders(self, symbol: str = '', autoCloseType: str = '', startTime: int = '', endTime: int = '',
                        limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-13

        Name         	Type	Mandatory	Description
        symbol       	str 	NO
        autoCloseType	str 	NO       	"LIQUIDATION": 强平单, "ADL": ADL减仓单.
        startTime    	int 	NO
        endTime      	int 	NO
        limit        	int 	NO       	Default 50; max 100.
        recvWindow   	int 	NO
        '''
        return self.send_request(*self.endpoints.get_forceOrders, **to_local(locals()))

    # 持仓ADL队列估算 (USER_DATA)
    def get_adlQuantile(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#adl-user_data

        Name      	Type	Mandatory	Description
        symbol    	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_adlQuantile, **to_local(locals()))

    # 用户手续费率 (USER_DATA)
    def get_commissionRate(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/delivery/cn/#user_data-14

        Name      	Type	Mandatory	Description
        symbol    	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_commissionRate, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 返佣
class _RebateEndpoints():
    get_taxQuery = ['https://api.binance.com/', 'GET', '/sapi/v1/rebate/taxQuery', True]  # 获取现货返佣历史记录 (USER_DATA)


class Rebate(Client):
    endpoints = _RebateEndpoints

    # 获取现货返佣历史记录 (USER_DATA)
    def get_taxQuery(self, startTime: int = '', endTime: int = '', page: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-117

        Name      	Type	Mandatory	Description
        startTime 	int 	NO
        endTime   	int 	NO
        page      	int 	NO       	默认 1
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_taxQuery, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


class _VIPLoansEndpoints():
    get_ongoing_orders = ['https://api.binance.com/','GET', '/sapi/v1/loan/vip/ongoing/orders', False]  # 查询VIP借币借款中订单 (USER_DATA)
    set_repay = ['https://api.binance.com/','POST', '/sapi/v1/loan/vip/repay', False]  # VIP借币还款 (TRADE)
    get_repay_history = ['https://api.binance.com/','GET', '/sapi/v1/loan/vip/repay/history', False]  # 查询VIP借币还款记录历史 (USER_DATA)


class VIPLoans(Client):
    endpoints = _VIPLoansEndpoints

    # 查询VIP借币借款中订单 (USER_DATA)
    def get_ongoing_orders(self, orderId: int = '', collateralAccountId: int = '', loanCoin: str = '',
                           collateralCoin: str = '', current: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#vip-user_data

        Name               	Type	Mandatory	Description
        orderId            	int 	NO
        collateralAccountId	int 	NO
        loanCoin           	str 	NO
        collateralCoin     	str 	NO
        current            	int 	NO       	当前查询页数，从1开始。默认值：1，最大：1000
        limit              	int 	NO       	默认值：10，最大：100
        recvWindow         	int 	NO
        '''
        return self.send_request(*self.endpoints.get_ongoing_orders, **to_local(locals()))

    # VIP借币还款 (TRADE)
    def set_repay(self, orderId: int = '', amount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#vip-trade

        Name      	Type            	Mandatory	Description
        orderId   	int             	YES
        amount    	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_repay, **to_local(locals()))

    # 查询VIP借币还款记录历史 (USER_DATA)
    def get_repay_history(self, orderId: int = '', loanCoin: str = '', startTime: int = '', endTime: int = '',
                          current: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#vip-user_data-2

        Name      	Type	Mandatory	Description
        orderId   	int 	NO
        loanCoin  	str 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        current   	int 	NO       	当前查询页数，从1开始。默认值：1，最大：1000
        limit     	int 	NO       	默认值：10，最大：100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_repay_history, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 矿池
class _MiningEndpoints():
    get_pub_algoList = ['https://api.binance.com/','GET', '/sapi/v1/mining/pub/algoList', True]  # 获取算法(MARKET_DATA)
    get_pub_coinList = ['https://api.binance.com/','GET', '/sapi/v1/mining/pub/coinList', True]  # 获取币种(MARKET_DATA)
    get_worker_detail = ['https://api.binance.com/','GET', '/sapi/v1/mining/worker/detail', True]  # 请求矿工列表明细 (USER_DATA)
    get_worker_list = ['https://api.binance.com/','GET', '/sapi/v1/mining/worker/list', True]  # 请求矿工列表 (USER_DATA)
    get_payment_list = ['https://api.binance.com/','GET', '/sapi/v1/mining/payment/list', True]  # 收益列表 (USER_DATA)
    get_payment_other = ['https://api.binance.com/','GET', '/sapi/v1/mining/payment/other', True]  # 其他收益列表 (USER_DATA)
    get_hash_transfer_config_details = ['https://api.binance.com/','GET', '/sapi/v1/mining/hash-transfer/config/details', True]  # 算力转让详情列表 (USER_DATA)
    get_hash_transfer_config_details_list = ['https://api.binance.com/','GET', '/sapi/v1/mining/hash-transfer/config/details/list', True]  # 算力转让列表 (USER_DATA)
    get_hash_transfer_profit_details = ['https://api.binance.com/','GET', '/sapi/v1/mining/hash-transfer/profit/details', True]  # 算力转让详情 (USER_DATA)
    set_hash_transfer_config = ['https://api.binance.com/','POST', '/sapi/v1/mining/hash-transfer/config', True]  # 算力转让请求 (USER_DATA)
    set_hash_transfer_config_cancel = ['https://api.binance.com/','POST', '/sapi/v1/mining/hash-transfer/config/cancel', True]  # 取消算力转让设置 (USER_DATA)
    get_statistics_user_status = ['https://api.binance.com/','GET', '/sapi/v1/mining/statistics/user/status', True]  # 统计列表 (USER_DATA)
    get_statistics_user_list = ['https://api.binance.com/','GET', '/sapi/v1/mining/statistics/user/list', True]  # 账号列表 (USER_DATA)
    get_payment_uid = ['https://api.binance.com/','GET', '/sapi/v1/mining/payment/uid', True]  # 矿池账户收益列表 (USER_DATA)


class Mining(Client):
    endpoints = _MiningEndpoints

    # 获取算法(MARKET_DATA)
    def get_pub_algoList(self):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-7
        '''
        return self.send_request(*self.endpoints.get_pub_algoList, **to_local(locals()))

    # 获取币种(MARKET_DATA)
    def get_pub_coinList(self):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data-8
        '''
        return self.send_request(*self.endpoints.get_pub_coinList, **to_local(locals()))

    # 请求矿工列表明细 (USER_DATA)
    def get_worker_detail(self, algo: str = '', userName: str = '', workerName: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-64

        Name      	Type	Mandatory	Description
        algo      	str 	YES
        userName  	str 	YES
        workerName	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_worker_detail, **to_local(locals()))

    # 请求矿工列表 (USER_DATA)
    def get_worker_list(self, algo: str = '', userName: str = '', pageIndex: int = '', sort: int = '',
                        sortColumn: int = '',
                        workerStatus: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-65

        Name        	Type	Mandatory	Description
        algo        	str 	YES
        userName    	str 	YES
        pageIndex   	int 	NO       	页码，为空默认第一页，从1开始
        sort        	int 	NO       	排序方向(为空默认为0): 0 正序，1 倒序
        sortColumn  	int 	NO
        workerStatus	int 	NO       	矿机状态(默认为0)：0 全部，1 有效， 2 无效， 3 失效
        recvWindow  	int 	NO
        '''
        return self.send_request(*self.endpoints.get_worker_list, **to_local(locals()))

    # 收益列表 (USER_DATA)
    def get_payment_list(self, algo: str = '', userName: str = '', coin: str = '', startDate: int = '',
                         endDate: int = '',
                         pageIndex: int = '', pageSize: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-66

        Name      	Type	Mandatory	Description
        algo      	str 	YES
        userName  	str 	YES
        coin      	str 	NO       	币种名称
        startDate 	int 	NO       	搜索日期 毫秒时间戳，同时为空查询所有
        endDate   	int 	NO       	搜索日期 毫秒时间戳，同时为空查询所有
        pageIndex 	int 	NO       	页码，为空默认第一页，从1开始
        pageSize  	int 	NO       	分页数量,最小10,最大200
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_payment_list, **to_local(locals()))

    # 其他收益列表 (USER_DATA)
    def get_payment_other(self, algo: str = '', userName: str = '', coin: str = '', startDate: int = '',
                          endDate: int = '',
                          pageIndex: int = '', pageSize: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-67

        Name      	Type	Mandatory	Description
        algo      	str 	YES
        userName  	str 	YES
        coin      	str 	NO       	币种名称
        startDate 	int 	NO       	搜索日期 毫秒时间戳，同时为空查询所有
        endDate   	int 	NO       	搜索日期 毫秒时间戳，同时为空查询所有
        pageIndex 	int 	NO       	页码，为空默认第一页，从1开始
        pageSize  	int 	NO       	分页数量,最小10,最大200
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_payment_other, **to_local(locals()))

    # 算力转让详情列表 (USER_DATA)
    def get_hash_transfer_config_details(self, pageIndex: int = '', pageSize: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-68

        Name      	Type	Mandatory	Description
        pageIndex 	int 	NO       	页码，为空默认第一页，从1开始
        pageSize  	int 	NO       	分页数量,最小10,最大200
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_hash_transfer_config_details, **to_local(locals()))

    # 算力转让列表 (USER_DATA)
    def get_hash_transfer_config_details_list(self, pageIndex: int = '', pageSize: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-69

        Name      	Type	Mandatory	Description
        pageIndex 	int 	NO       	页码，为空默认第一页，从1开始
        pageSize  	int 	NO       	分页数量,最小10,最大200
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_hash_transfer_config_details_list, **to_local(locals()))

    # 算力转让详情 (USER_DATA)
    def get_hash_transfer_profit_details(self, configId: int = '', pageIndex: int = '', pageSize: int = '',
                                         recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-70

        Name      	Type	Mandatory	Description
        configId  	int 	YES
        pageIndex 	int 	NO       	页码，为空默认第一页，从1开始
        pageSize  	int 	NO       	分页数量,最小10,最大200
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_hash_transfer_profit_details, **to_local(locals()))

    # 算力转让请求 (USER_DATA)
    def set_hash_transfer_config(self, userName: str = '', algo: str = '', endDate: int = '', startDate: int = '',
                                 toPoolUser: str = '', hashRate: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-71

        Name      	Type	Mandatory	Description
        userName  	str 	YES
        algo      	str 	YES
        endDate   	int 	YES
        startDate 	int 	YES
        toPoolUser	str 	YES
        hashRate  	int 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_hash_transfer_config, **to_local(locals()))

    # 取消算力转让设置 (USER_DATA)
    def set_hash_transfer_config_cancel(self, configId: int = '', userName: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-72

        Name      	Type	Mandatory	Description
        configId  	int 	YES
        userName  	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_hash_transfer_config_cancel, **to_local(locals()))

    # 统计列表 (USER_DATA)
    def get_statistics_user_status(self, algo: str = '', userName: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-73

        Name      	Type	Mandatory	Description
        algo      	str 	YES
        userName  	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_statistics_user_status, **to_local(locals()))

    # 账号列表 (USER_DATA)
    def get_statistics_user_list(self, algo: str = '', userName: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-74

        Name      	Type	Mandatory	Description
        algo      	str 	YES
        userName  	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_statistics_user_list, **to_local(locals()))

    # 矿池账户收益列表 (USER_DATA)
    def get_payment_uid(self, algo: str = '', startDate: int = '', endDate: int = '', pageIndex: int = '',
                        pageSize: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-75

        Name      	Type	Mandatory	Description
        algo      	str 	YES
        startDate 	int 	NO       	搜索日期 毫秒时间戳，同时为空查询所有
        endDate   	int 	NO       	搜索日期 毫秒时间戳，同时为空查询所有
        pageIndex 	int 	NO       	页码，为空默认第一页，从1开始
        pageSize  	int 	NO       	分页数量,最小10,最大200
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_payment_uid, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 行情信息
class _UMMarketEndpoints():
    get_ping = ['https://fapi.binance.com/','GET', '/fapi/v1/ping', False]  # 测试服务器连通性 PING
    get_time = ['https://fapi.binance.com/','GET', '/fapi/v1/time', False]  # 获取服务器时间
    get_exchangeInfo = ['https://fapi.binance.com/','GET', '/fapi/v1/exchangeInfo', False]  # 获取交易规则和交易对
    get_depth = ['https://fapi.binance.com/','GET', '/fapi/v1/depth', False]  # 深度信息
    get_trades = ['https://fapi.binance.com/','GET', '/fapi/v1/trades', False]  # 近期成交
    get_historicalTrades = ['https://fapi.binance.com/','GET', '/fapi/v1/historicalTrades', False]  # 查询历史成交(MARKET_DATA)
    get_aggTrades = ['https://fapi.binance.com/','GET', '/fapi/v1/aggTrades', False]  # 近期成交(归集)
    get_klines = ['https://fapi.binance.com/','GET', '/fapi/v1/klines', False]  # K线数据
    get_continuousKlines = ['https://fapi.binance.com/','GET', '/fapi/v1/continuousKlines', False]  # 连续合约K线数据
    get_indexPriceKlines = ['https://fapi.binance.com/','GET', '/fapi/v1/indexPriceKlines', False]  # 价格指数K线数据
    get_markPriceKlines = ['https://fapi.binance.com/','GET', '/fapi/v1/markPriceKlines', False]  # 标记价格K线数据
    get_premiumIndex = ['https://fapi.binance.com/','GET', '/fapi/v1/premiumIndex', False]  # 最新标记价格和资金费率
    get_fundingRate = ['https://fapi.binance.com/','GET', '/fapi/v1/fundingRate', False]  # 查询资金费率历史
    get_ticker_24hr = ['https://fapi.binance.com/','GET', '/fapi/v1/ticker/24hr', False]  # 24hr价格变动情况
    get_ticker_price = ['https://fapi.binance.com/','GET', '/fapi/v1/ticker/price', False]  # 最新价格
    get_ticker_bookTicker = ['https://fapi.binance.com/','GET', '/fapi/v1/ticker/bookTicker', False]  # 当前最优挂单
    get_openInterest = ['https://fapi.binance.com/','GET', '/fapi/v1/openInterest', False]  # 获取未平仓合约数
    get_openInterestHist = ['https://fapi.binance.com/','GET', '/futures/data/openInterestHist', False]  # 合约持仓量
    get_topLongShortAccountRatio = ['https://fapi.binance.com/','GET', '/futures/data/topLongShortAccountRatio', False]  # 大户账户数多空比
    get_topLongShortPositionRatio = ['https://fapi.binance.com/','GET', '/futures/data/topLongShortPositionRatio', False]  # 大户持仓量多空比
    get_globalLongShortAccountRatio = ['https://fapi.binance.com/','GET', '/futures/data/globalLongShortAccountRatio', False]  # 多空持仓人数比
    get_takerlongshortRatio = ['https://fapi.binance.com/','GET', '/futures/data/takerlongshortRatio', False]  # 合约主动买卖量
    get_lvtKlines = ['https://fapi.binance.com/','GET', '/fapi/v1/lvtKlines', False]  # 杠杆代币历史净值K线
    get_indexInfo = ['https://fapi.binance.com/','GET', '/fapi/v1/indexInfo', False]  # 综合指数交易对信息
    get_assetIndex = ['https://fapi.binance.com/','GET', '/fapi/v1/assetIndex', False]  # 多资产模式资产汇率指数


class UMMarket(Client):
    endpoints = _UMMarketEndpoints


    # 测试服务器连通性 PING
    def get_ping(self):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#ping
        '''
        return self.send_request(*self.endpoints.get_ping, **to_local(locals()))


    # 获取服务器时间
    def get_time(self):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#3f1907847c
        '''
        return self.send_request(*self.endpoints.get_time, **to_local(locals()))


    # 获取交易规则和交易对
    def get_exchangeInfo(self):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#0f3f2d5ee7
        '''
        return self.send_request(*self.endpoints.get_exchangeInfo, **to_local(locals()))


    # 深度信息
    def get_depth(self, symbol: str = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#38a975b802

        Name  	Type	Mandatory	Description
        symbol	str 	YES      	交易对
        limit 	int 	NO       	默认 500; 可选值:[5, 10, 20, 50, 100, 500, 1000]
        '''
        return self.send_request(*self.endpoints.get_depth, **to_local(locals()))


    # 近期成交
    def get_trades(self, symbol: str = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#404aacd9b3

        Name  	Type	Mandatory	Description
        symbol	str 	YES      	交易对
        limit 	int 	NO       	默认:500，最大1000
        '''
        return self.send_request(*self.endpoints.get_trades, **to_local(locals()))


    # 查询历史成交(MARKET_DATA)
    def get_historicalTrades(self, symbol: str = '', limit: int = '', fromId: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#market_data

        Name  	Type	Mandatory	Description
        symbol	str 	YES      	交易对
        limit 	int 	NO       	默认值:500 最大值:1000.
        fromId	int 	NO       	从哪一条成交id开始返回. 缺省返回最近的成交记录
        '''
        return self.send_request(*self.endpoints.get_historicalTrades, **to_local(locals()))


    # 近期成交(归集)
    def get_aggTrades(self, symbol: str = '', fromId: int = '', startTime: int = '', endTime: int = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#c59e471e81

        Name     	Type	Mandatory	Description
        symbol   	str 	YES      	交易对
        fromId   	int 	NO       	从包含fromID的成交开始返回结果
        startTime	int 	NO       	从该时刻之后的成交记录开始返回结果
        endTime  	int 	NO       	返回该时刻为止的成交记录
        limit    	int 	NO       	默认 500; 最大 1000.
        '''
        return self.send_request(*self.endpoints.get_aggTrades, **to_local(locals()))


    # K线数据
    def get_klines(self, symbol: str = '', interval: str = '', startTime: int = '', endTime: int = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#k

        Name     	Type	Mandatory	Description
        symbol   	str 	YES      	交易对
        interval 	str 	YES      	时间间隔
        startTime	int 	NO       	起始时间
        endTime  	int 	NO       	结束时间
        limit    	int 	NO       	默认值:500 最大值:1500.
        '''
        return self.send_request(*self.endpoints.get_klines, **to_local(locals()))


    # 连续合约K线数据
    def get_continuousKlines(self, pair: str = '', contractType: str = '', interval: str = '', startTime: int = '',
                             endTime: int = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#k-2

        Name        	Type	Mandatory	Description
        pair        	str 	YES      	标的交易对
        contractType	str 	YES      	合约类型
        interval    	str 	YES      	时间间隔
        startTime   	int 	NO       	起始时间
        endTime     	int 	NO       	结束时间
        limit       	int 	NO       	默认值:500 最大值:1500
        '''
        return self.send_request(*self.endpoints.get_continuousKlines, **to_local(locals()))


    # 价格指数K线数据
    def get_indexPriceKlines(self, pair: str = '', interval: str = '', startTime: int = '', endTime: int = '',
                             limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#k-3

        Name     	Type	Mandatory	Description
        pair     	str 	YES      	标的交易对
        interval 	str 	YES      	时间间隔
        startTime	int 	NO       	起始时间
        endTime  	int 	NO       	结束时间
        limit    	int 	NO       	默认值:500 最大值:1500
        '''
        return self.send_request(*self.endpoints.get_indexPriceKlines, **to_local(locals()))


    # 标记价格K线数据
    def get_markPriceKlines(self, symbol: str = '', interval: str = '', startTime: int = '', endTime: int = '',
                            limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#k-4

        Name     	Type	Mandatory	Description
        symbol   	str 	YES      	交易对
        interval 	str 	YES      	时间间隔
        startTime	int 	NO       	起始时间
        endTime  	int 	NO       	结束时间
        limit    	int 	NO       	默认值:500 最大值:1500
        '''
        return self.send_request(*self.endpoints.get_markPriceKlines, **to_local(locals()))


    # 最新标记价格和资金费率
    def get_premiumIndex(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#69f9b0b2f3

        Name  	Type	Mandatory	Description
        symbol	str 	NO       	交易对
        '''
        return self.send_request(*self.endpoints.get_premiumIndex, **to_local(locals()))


    # 查询资金费率历史
    def get_fundingRate(self, symbol: str = '', startTime: int = '', endTime: int = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#31dbeb24c4

        Name     	Type	Mandatory	Description
        symbol   	str 	NO       	交易对
        startTime	int 	NO       	起始时间
        endTime  	int 	NO       	结束时间
        limit    	int 	NO       	默认值:100 最大值:1000
        '''
        return self.send_request(*self.endpoints.get_fundingRate, **to_local(locals()))


    # 24hr价格变动情况
    def get_ticker_24hr(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#24hr

        Name  	Type	Mandatory	Description
        symbol	str 	NO       	交易对
        '''
        return self.send_request(*self.endpoints.get_ticker_24hr, **to_local(locals()))


    # 最新价格
    def get_ticker_price(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#8ff46b58de

        Name  	Type	Mandatory	Description
        symbol	str 	NO       	交易对
        '''
        return self.send_request(*self.endpoints.get_ticker_price, **to_local(locals()))


    # 当前最优挂单
    def get_ticker_bookTicker(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#5393cd07b4

        Name  	Type	Mandatory	Description
        symbol	str 	NO       	交易对
        '''
        return self.send_request(*self.endpoints.get_ticker_bookTicker, **to_local(locals()))


    # 获取未平仓合约数
    def get_openInterest(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#f6cc22e496

        Name  	Type	Mandatory	Description
        symbol	str 	YES      	交易对
        '''
        return self.send_request(*self.endpoints.get_openInterest, **to_local(locals()))


    # 合约持仓量
    def get_openInterestHist(self, symbol: str = '', period: str = '', limit: int = '', startTime: int = '',
                             endTime: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#c1c9193984

        Name     	Type	Mandatory	Description
        symbol   	str 	YES
        period   	str 	YES      	"5m","15m","30m","1h","2h","4h","6h","12h","1d"
        limit    	int 	NO       	default 30, max 500
        startTime	int 	NO
        endTime  	int 	NO
        '''
        return self.send_request(*self.endpoints.get_openInterestHist, **to_local(locals()))


    # 大户账户数多空比
    def get_topLongShortAccountRatio(self, symbol: str = '', period: str = '', limit: int = '', startTime: int = '',
                                     endTime: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#4d050dd845

        Name     	Type	Mandatory	Description
        symbol   	str 	YES
        period   	str 	YES      	"5m","15m","30m","1h","2h","4h","6h","12h","1d"
        limit    	int 	NO       	default 30, max 500
        startTime	int 	NO
        endTime  	int 	NO
        '''
        return self.send_request(*self.endpoints.get_topLongShortAccountRatio, **to_local(locals()))


    # 大户持仓量多空比
    def get_topLongShortPositionRatio(self, symbol: str = '', period: str = '', limit: int = '', startTime: int = '',
                                      endTime: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#269e531011

        Name     	Type	Mandatory	Description
        symbol   	str 	YES
        period   	str 	YES      	"5m","15m","30m","1h","2h","4h","6h","12h","1d"
        limit    	int 	NO       	default 30, max 500
        startTime	int 	NO
        endTime  	int 	NO
        '''
        return self.send_request(*self.endpoints.get_topLongShortPositionRatio, **to_local(locals()))


    # 多空持仓人数比
    def get_globalLongShortAccountRatio(self, symbol: str = '', period: str = '', limit: int = '', startTime: int = '',
                                        endTime: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#4d55ab5a20

        Name     	Type	Mandatory	Description
        symbol   	str 	YES
        period   	str 	YES      	"5m","15m","30m","1h","2h","4h","6h","12h","1d"
        limit    	int 	NO       	default 30, max 500
        startTime	int 	NO
        endTime  	int 	NO
        '''
        return self.send_request(*self.endpoints.get_globalLongShortAccountRatio, **to_local(locals()))


    # 合约主动买卖量
    def get_takerlongshortRatio(self, symbol: str = '', period: str = '', limit: int = '', startTime: int = '',
                                endTime: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#249be3ac39

        Name     	Type	Mandatory	Description
        symbol   	str 	YES
        period   	str 	YES      	"5m","15m","30m","1h","2h","4h","6h","12h","1d"
        limit    	int 	NO       	default 30, max 500
        startTime	int 	NO
        endTime  	int 	NO
        '''
        return self.send_request(*self.endpoints.get_takerlongshortRatio, **to_local(locals()))


    # 杠杆代币历史净值K线
    def get_lvtKlines(self, symbol: str = '', interval: str = '', startTime: int = '', endTime: int = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#k-5

        Name     	Type	Mandatory	Description
        symbol   	str 	YES      	token name, e.g. "BTCDOWN", "BTCUP"
        interval 	str 	YES
        startTime	int 	NO
        endTime  	int 	NO
        limit    	int 	NO       	默认 500, 最大 1000
        '''
        return self.send_request(*self.endpoints.get_lvtKlines, **to_local(locals()))


    # 综合指数交易对信息
    def get_indexInfo(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#669b9d47c2

        Name  	Type	Mandatory	Description
        symbol	str 	NO
        '''
        return self.send_request(*self.endpoints.get_indexInfo, **to_local(locals()))


    # 多资产模式资产汇率指数
    def get_assetIndex(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/futures/cn/#c30dcbaf8a

        Name  	Type	Mandatory	Description
        symbol	str 	NO       	资产对
        '''
        return self.send_request(*self.endpoints.get_assetIndex, **to_local(locals()))




from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 账户与交易
class _EOAccountTradeEndpoints():
    get_account = ['https://eapi.binance.com/','GET', '/eapi/v1/account', True]  # 账户信息 (TRADE)
    set_transfer = ['https://eapi.binance.com/','POST', '/eapi/v1/transfer', True]  # 资金划转 (TRADE)
    set_order = ['https://eapi.binance.com/','POST', '/eapi/v1/order', True]  # 下单 (TRADE)
    set_batchOrders = ['https://eapi.binance.com/','POST', '/eapi/v1/batchOrders', True]  # 批量下单 (TRADE)
    cancel_order = ['https://eapi.binance.com/','DELETE', '/eapi/v1/order', True]  # 撤销订单 (TRADE)
    cancel_batchOrders = ['https://eapi.binance.com/','DELETE', '/eapi/v1/batchOrders', True]  # 批量撤销订单 (TRADE)
    cancel_allOpenOrders = ['https://eapi.binance.com/','DELETE', '/eapi/v1/allOpenOrders', True]  # 撤销单交易对全部订单 (TRADE)
    cancel_allOpenOrdersByUnderlying = ['https://eapi.binance.com/','DELETE', '/eapi/v1/allOpenOrdersByUnderlying', True]  # 撤销特定标的全部订单 (TRADE)
    get_openOrders = ['https://eapi.binance.com/','GET', '/eapi/v1/openOrders', True]  # 查询当前挂单 (USER_DATA)
    get_historyOrders = ['https://eapi.binance.com/','GET', '/eapi/v1/historyOrders', True]  # 查询历史订单 (USER_DATA)
    get_position = ['https://eapi.binance.com/','GET', '/eapi/v1/position', True]  # 仓位信息 (USER_DATA)
    get_userTrades = ['https://eapi.binance.com/','GET', '/eapi/v1/userTrades', True]  # 账户成交历史 (USER_DATA)
    get_exerciseRecord = ['https://eapi.binance.com/','GET', '/eapi/v1/exerciseRecord', True]  # 用户行权历史(USER_DATA)
    get_bill = ['https://eapi.binance.com/','GET', '/eapi/v1/bill', True]  # 获取账户资金流水(USER_DATA)


class EOAccountTrade(Client):
    endpoints = _EOAccountTradeEndpoints

    # 账户信息 (TRADE)
    def get_account(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_account, **to_local(locals()))

    # 资金划转 (TRADE)
    def set_transfer(self, currency: str = '', type: str = '', amount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-2

        Name      	Type            	Mandatory	Description
        currency  	str             	YES      	资产类型, e.g USDT
        type      	str             	YES      	IN: 从现货钱包转到期权钱包 OUT: 从期权钱包转到现货钱包
        amount    	Union[float,int]	YES      	数量如100
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_transfer, **to_local(locals()))

    # 下单 (TRADE)
    def set_order(self, symbol: str = '', side: str = '', type: str = '', quantity: Union[float, int] = '',
                  price: Union[float, int] = '', timeInForce: str = '', reduceOnly: str = '', postOnly: str = '',
                  newOrderRespType: str = '', clientOrderId: str = '', isMmp: bool = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-3

        Name            	Type            	Mandatory	Description
        symbol          	str             	YES      	交易对
        side            	str             	YES
        type            	str             	YES
        quantity        	Union[float,int]	YES      	下单数量
        price           	Union[float,int]	NO       	委托价格
        timeInForce     	str             	NO       	有效时间
        reduceOnly      	str             	NO
        postOnly        	str             	NO
        newOrderRespType	str             	NO       	"ACK", "RESULT", 默认 "ACK"
        clientOrderId   	str             	NO       	用户自定义的订单号，不可以重复出现在挂单中。如空缺系统会自动赋值。
        isMmp           	bool            	NO       	是否为MMP订单true/false
        recvWindow      	int             	NO
        '''
        return self.send_request(*self.endpoints.set_order, **to_local(locals()))

    # 批量下单 (TRADE)
    def set_batchOrders(self, orders: list = '', recvWindow: int = '', symbol: str = '', side: str = '', type: str = '',
                        quantity: Union[float, int] = '', price: Union[float, int] = '', timeInForce: str = '',
                        reduceOnly: str = '', postOnly: str = '', newOrderRespType: str = '', clientOrderId: str = '',
                        isMmp: bool = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-4

        Name            	Type            	Mandatory	Description
        orders          	list            	YES      	订单列表，最多支持5个订单
        recvWindow      	int             	NO
        symbol          	str             	YES      	交易对
        side            	str             	YES
        type            	str             	YES
        quantity        	Union[float,int]	YES      	下单数量
        price           	Union[float,int]	NO       	委托价格
        timeInForce     	str             	NO       	有效时间
        reduceOnly      	str             	NO
        postOnly        	str             	NO
        newOrderRespType	str             	NO       	"ACK", "RESULT", 默认 "ACK"
        clientOrderId   	str             	NO
        isMmp           	bool            	NO       	是否为MMP订单true/false
        '''
        return self.send_request(*self.endpoints.set_batchOrders, **to_local(locals()))

    # 撤销订单 (TRADE)
    def cancel_order(self, symbol: str = '', orderId: int = '', clientOrderId: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-5

        Name         	Type	Mandatory	Description
        symbol       	str 	YES      	交易对
        orderId      	int 	NO       	系统订单号
        clientOrderId	str 	NO       	用户自定义的订单号
        recvWindow   	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_order, **to_local(locals()))

    # 批量撤销订单 (TRADE)
    def cancel_batchOrders(self, symbol: str = '', orderIds: list = '', clientOrderIds: list = '',
                           recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-6

        Name          	Type        	Mandatory	Description
        symbol        	str         	YES      	交易对
        orderIds      	LIST<LONG>  	NO
        clientOrderIds	LIST<STRING>	NO
        recvWindow    	int         	NO
        '''
        return self.send_request(*self.endpoints.cancel_batchOrders, **to_local(locals()))

    # 撤销单交易对全部订单 (TRADE)
    def cancel_allOpenOrders(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-7

        Name      	Type	Mandatory	Description
        symbol    	str 	YES      	交易对
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_allOpenOrders, **to_local(locals()))

    # 撤销特定标的全部订单 (TRADE)
    def cancel_allOpenOrdersByUnderlying(self, underlying: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-8

        Name      	Type	Mandatory	Description
        underlying	str 	YES      	标的资产如BTCUSDT
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.cancel_allOpenOrdersByUnderlying, **to_local(locals()))

    # 查询当前挂单 (USER_DATA)
    def get_openOrders(self, symbol: str = '', orderId: int = '', startTime: int = '', endTime: int = '',
                       limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#user_data

        Name      	Type	Mandatory	Description
        symbol    	str 	NO       	交易对(不传返回所有订单)
        orderId   	int 	NO       	只返回此orderID及之后的订单，缺省返回最近的订单
        startTime 	int 	NO       	开始时间
        endTime   	int 	NO       	结束时间
        limit     	int 	NO       	返回数量，默认100 最大1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_openOrders, **to_local(locals()))

    # 查询历史订单 (USER_DATA)
    def get_historyOrders(self, symbol: str = '', orderId: int = '', startTime: int = '', endTime: int = '',
                          limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#user_data-2

        Name      	Type	Mandatory	Description
        symbol    	str 	YES      	交易对
        orderId   	int 	NO       	只返回此orderID及之后的订单，缺省返回最近的订单
        startTime 	int 	NO       	起始时间
        endTime   	int 	NO       	结束时间
        limit     	int 	NO       	返回的结果集数量 默认值:500 最大值:1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_historyOrders, **to_local(locals()))

    # 仓位信息 (USER_DATA)
    def get_position(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#user_data-3

        Name      	Type	Mandatory	Description
        symbol    	str 	NO       	交易对如BTC-200730-9000-C
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_position, **to_local(locals()))

    # 账户成交历史 (USER_DATA)
    def get_userTrades(self, symbol: str = '', fromId: int = '', startTime: int = '', endTime: int = '',
                       limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#user_data-4

        Name      	Type	Mandatory	Description
        symbol    	str 	NO       	交易对
        fromId    	int 	NO       	返回该fromId及之后的成交，缺省返回最近的成交
        startTime 	int 	NO       	起始时间
        endTime   	int 	NO       	结束时间
        limit     	int 	NO       	返回的结果集数量 默认值:100 最大值:1000.
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_userTrades, **to_local(locals()))

    # 用户行权历史(USER_DATA)
    def get_exerciseRecord(self, symbol: str = '', startTime: int = '', endTime: int = '', limit: int = '',
                           recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#user_data-5

        Name      	Type	Mandatory	Description
        symbol    	str 	NO       	交易对如 BTC-200730-9000-C
        startTime 	int 	NO       	起始时间如1593511200000
        endTime   	int 	NO       	结束时间如1593512200000
        limit     	int 	NO       	默认1000, 最大1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_exerciseRecord, **to_local(locals()))

    # 获取账户资金流水(USER_DATA)
    def get_bill(self, currency: str = '', recordId: int = '', startTime: int = '', endTime: int = '', limit: int = '',
                 recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#user_data-6

        Name      	Type	Mandatory	Description
        currency  	str 	YES      	资产类型如USDT
        recordId  	int 	NO       	返回该recordId及之后的成交，缺省返回最近的成交
        startTime 	int 	NO       	起始时间如1593511200000
        endTime   	int 	NO       	结束时间如1593512200000
        limit     	int 	NO       	默认100 最大1000
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_bill, **to_local(locals()))



'''
币本位合约
https://binance-docs.github.io/apidocs/delivery/cn/#185368440e
'''

from .cm_market import CMMarket  # 行情信息
from .cm_account_trade import CMAccountTrade  # 账户与交易
from .cm_portfolio_margin import CMPortfolioMargin  # 统一账户

Market = CMMarket
AccountTrade = CMAccountTrade
PortfolioMargin = CMPortfolioMargin


class CM():
    def __init__(self, key='', secret=''):
        self.market = Market(key=key, secret=secret)
        self.accountTrade = AccountTrade(key=key, secret=secret)
        self.portfolioMargin = PortfolioMargin(key=key, secret=secret)



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 市商
class _EOMarketMakerEndpoints():
    get_marginAccount = ['https://eapi.binance.com/','GET', '/eapi/v1/marginAccount', True]  # 保证金账户信息
    set_mmpSet = ['https://eapi.binance.com/','POST', '/eapi/v1/mmpSet', True]  # 设置MMP规则
    get_mmpSet = ['https://eapi.binance.com/','GET', '/eapi/v1/mmpSet', True]  # 获取MMP规则
    set_mmpReset = ['https://eapi.binance.com/','POST', '/eapi/v1/mmpReset', True]  # 重置MMP状态
    set_countdownCancelAll = ['https://eapi.binance.com/','POST', '/eapi/v1/countdownCancelAll', True]  # 设置倒计时取消所有订单配置 (TRADE)
    get_countdownCancelAll = ['https://eapi.binance.com/','GET', '/eapi/v1/countdownCancelAll', True]  # 获得倒计时自动取消所有订单配置 (TRADE)
    set_countdownCancelAllHeartBeat = ['https://eapi.binance.com/','POST', '/eapi/v1/countdownCancelAllHeartBeat', True]  #重置倒计时取消所有订单心跳 (TRADE)


class EOMarketMaker(Client):
    endpoints = _EOMarketMakerEndpoints

    # 保证金账户信息
    def get_marginAccount(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#f5972c1906

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_marginAccount, **to_local(locals()))

    # 设置MMP规则
    def set_mmpSet(self, underlying: str = '', windowTimeInMilliseconds: int = '', frozenTimeInMilliseconds: int = '',
                   qtyLimit: Union[float, int] = '', deltaLimit: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#mmp

        Name                    	Type            	Mandatory	Description
        underlying              	str             	YES      	标的资产如BTCUSDT
        windowTimeInMilliseconds	int             	YES      	MMP监控时间窗口（毫秒），在(0,5000]之间
        frozenTimeInMilliseconds	int             	NO       	MMP冻结时间（毫秒），设置为0后代表账号为冻结状态，需要手动重置
        qtyLimit                	Union[float,int]	NO       	数量限制
        deltaLimit              	Union[float,int]	NO       	净delta限制
        recvWindow              	int             	NO
        '''
        return self.send_request(*self.endpoints.set_mmpSet, **to_local(locals()))

    # 获取MMP规则
    def get_mmpSet(self, underlying: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#mmp-2

        Name      	Type	Mandatory	Description
        underlying	str 	YES      	标的资产如BTCUSDT
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_mmpSet, **to_local(locals()))

    # 重置MMP状态
    def set_mmpReset(self, underlying: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#mmp-3

        Name      	Type	Mandatory	Description
        underlying	str 	YES      	标的资产如BTCUSDT
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_mmpReset, **to_local(locals()))

    # 设置倒计时取消所有订单配置 (TRADE)
    def set_countdownCancelAll(self, underlying: str = '', countdownTime: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-9

        Name         	Type	Mandatory	Description
        underlying   	str 	YES      	期权标的资产， 如 BTCUSDT
        countdownTime	int 	YES      	以毫秒计量倒计时长 (1000代表1秒)。 设为0时关闭倒计时。最小设为5000（负值无效）
        recvWindow   	int 	NO
        '''
        return self.send_request(*self.endpoints.set_countdownCancelAll, **to_local(locals()))

    # 获得倒计时自动取消所有订单配置 (TRADE)
    def get_countdownCancelAll(self, underlying: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-10

        Name      	Type	Mandatory	Description
        underlying	str 	YES      	期权标的资产， 如BTCUSDT
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_countdownCancelAll, **to_local(locals()))

    # 重置倒计时取消所有订单心跳 (TRADE)
    def set_countdownCancelAllHeartBeat(self, underlyings: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/voptions/cn/#trade-11

        Name       	Type	Mandatory	Description
        underlyings	str 	YES      	期权标的资产， 如BTCUSDT
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.set_countdownCancelAllHeartBeat, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union

# staking接口
class _StakingEndpoints():
    get_productList = ['https://api.binance.com/','GET', '/sapi/v1/staking/productList', True]  # 查询Staking产品列表(USER_DATA)
    set_purchase = ['https://api.binance.com/','POST', '/sapi/v1/staking/purchase', True]  # 申购锁仓产品(USER_DATA)
    set_redeem = ['https://api.binance.com/','POST', '/sapi/v1/staking/redeem', True]  # 赎回锁仓产品(USER_DATA)
    get_position = ['https://api.binance.com/','GET', '/sapi/v1/staking/position', True]  # 查看个人持仓(USER_DATA)
    get_stakingRecord = ['https://api.binance.com/','GET', '/sapi/v1/staking/stakingRecord', True]  # 查看Staking历史记录(USER_DATA)
    set_setAutoStaking = ['https://api.binance.com/','POST', '/sapi/v1/staking/setAutoStaking', True]  # 设置自动续期(USER_DATA)
    get_personalLeftQuota = ['https://api.binance.com/','GET', '/sapi/v1/staking/personalLeftQuota', True]  # 查询Staking个人剩余额度(USER_DATA)A


class Staking(Client):
    endpoints = _StakingEndpoints

    # 查询Staking产品列表(USER_DATA)
    def get_productList(self, product: str = '', asset: str = '', current: int = '', size: int = '',
                        recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#staking-user_data

        Name      	Type	Mandatory	Description
        product   	str 	YES      	"STAKING" 是Staking, "F_DEFI" 是DeFi活期挖矿, "L_DEFI" 是DeFi定期挖矿
        asset     	str 	NO
        current   	int 	NO       	当前查询页。 开始值 1，默认:1
        size      	int 	NO       	默认：10，最大：100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_productList, **to_local(locals()))

    # 申购锁仓产品(USER_DATA)
    def set_purchase(self, product: str = '', productId: str = '', amount: Union[float, int] = '', renewable: str = '',
                     recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-60

        Name      	Type            	Mandatory	Description
        product   	str             	YES      	"STAKING" 是Staking, "F_DEFI" 是DeFi活期挖矿, "L_DEFI" 是DeFi定期挖矿
        productId 	str             	YES
        amount    	Union[float,int]	YES
        renewable 	str             	NO       	true或者false，默认为false。仅在产品为 "STAKING" 或 "L_DEFI"时生效
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_purchase, **to_local(locals()))

    # 赎回锁仓产品(USER_DATA)
    def set_redeem(self, product: str = '', positionId: str = '', productId: str = '', amount: Union[float, int] = '',
                   recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-61

        Name      	Type            	Mandatory	Description
        product   	str             	YES      	"STAKING" 是Staking, "F_DEFI" 是DeFi活期挖矿, "L_DEFI" 是DeFi定期挖矿
        positionId	str             	NO       	"1234", 当产品为"STAKING" 或 "L_DEFI"时为必填项
        productId 	str             	YES
        amount    	Union[float,int]	NO       	当产品为"F_DEFI" 时为必填项
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_redeem, **to_local(locals()))

    # 查看个人持仓(USER_DATA)
    def get_position(self, product: str = '', productId: str = '', asset: str = '', current: int = '', size: int = '',
                     recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-62

        Name      	Type	Mandatory	Description
        product   	str 	YES      	"STAKING" 是Staking, "F_DEFI" 是DeFi活期挖矿, "L_DEFI" 是DeFi定期挖矿
        productId 	str 	NO
        asset     	str 	NO
        current   	int 	NO       	当前查询页。 开始值 1， 默认:1
        size      	int 	NO       	默认：10，最大：100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_position, **to_local(locals()))

    # 查看Staking历史记录(USER_DATA)
    def get_stakingRecord(self, product: str = '', txnType: str = '', asset: str = '', startTime: int = '',
                          endTime: int = '', current: int = '', size: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#staking-user_data-2

        Name      	Type	Mandatory	Description
        product   	str 	YES      	"STAKING" 是Staking, "F_DEFI" 是DeFi活期挖矿, "L_DEFI" 是DeFi定期挖矿
        txnType   	str 	YES      	申购："SUBSCRIPTION", 赎回："REDEMPTION", 利息："INTEREST"
        asset     	str 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        current   	int 	NO       	当前查询页。 开始值 1, 默认:1
        size      	int 	NO       	默认：10，最大：100
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_stakingRecord, **to_local(locals()))

    # 设置自动续期(USER_DATA)
    def set_setAutoStaking(self, product: str = '', positionId: str = '', renewable: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-63

        Name      	Type	Mandatory	Description
        product   	str 	YES      	"STAKING" 是Staking, "L_DEFI" 是DeFi定期挖矿
        positionId	str 	YES
        renewable 	str 	YES      	true 或者 false
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_setAutoStaking, **to_local(locals()))

    def get_personalLeftQuota(self, product: str = '', productId: str = '', recvWindow: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#staking-user_data-3

        Name      	Type	Mandatory	Description
        product   	str 	YES      	"STAKING" 是Staking, "F_DEFI" 是DeFi活期挖矿, "L_DEFI" 是DeFi定期挖矿
        productId	str 	YES
        recvWindow 	str 	YES
        '''
        return self.send_request(*self.endpoints.get_personalLeftQuota, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union

# 行情信息
class _SPOTMarketEndpoints():
    get_ping = ['https://api.binance.com/', 'GET', '/api/v3/ping', False]  # 测试服务器连通性
    get_time = ['https://api.binance.com/', 'GET', '/api/v3/time', False]  # 获取服务器时间
    get_exchangeInfo = ['https://api.binance.com/', 'GET', '/api/v3/exchangeInfo', False]  # 交易规范信息
    get_depth = ['https://api.binance.com/', 'GET', '/api/v3/depth', False]  # 深度信息
    get_trades = ['https://api.binance.com/', 'GET', '/api/v3/trades', False]  # 近期成交列表
    get_historicalTrades = ['https://api.binance.com/', 'GET', '/api/v3/historicalTrades', False]  # 查询历史成交 (MARKET_DATA)
    get_aggTrades = ['https://api.binance.com/', 'GET', '/api/v3/aggTrades', False]  # 近期成交(归集)
    get_klines = ['https://api.binance.com/', 'GET', '/api/v3/klines', False]  # K线数据
    get_avgPrice = ['https://api.binance.com/', 'GET', '/api/v3/avgPrice', False]  # 当前平均价格
    get_uiKlines = ['https://api.binance.com/', 'GET', '/api/v3/uiKlines', False]  # UIK线数据
    get_ticker_24hr = ['https://api.binance.com/', 'GET', '/api/v3/ticker/24hr', False]  # 24hr 价格变动情况
    get_ticker_price = ['https://api.binance.com/', 'GET', '/api/v3/ticker/price', False]  # 最新价格
    get_ticker_bookTicker = ['https://api.binance.com/', 'GET', '/api/v3/ticker/bookTicker', False]  # 当前最优挂单
    get_ticker = ['https://api.binance.com/', 'GET', '/api/v3/ticker', False]  # 滚动窗口价格变动统计


class SPOTMarket(Client):
    endpoints = _SPOTMarketEndpoints

    # 测试服务器连通性
    def get_ping(self):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#16073bbcf1
        '''
        return self.send_request(*self.endpoints.get_ping, **to_local(locals()))

    # 获取服务器时间
    def get_time(self):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#3f1907847c
        '''
        return self.send_request(*self.endpoints.get_time, **to_local(locals()))

    # 交易规范信息
    def get_exchangeInfo(self, symbol: str = '', symbols: list = [], permissions: Union[list, str] = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#e7746f7d60
        Name     	Type	Mandatory	Description
        symbol   	str 	YES         单个交易对
        symbols   	list 	NO          多个交易对
        permissions	int 	NO          交易权限
        '''
        return self.send_request(*self.endpoints.get_exchangeInfo, **to_local(locals()))

    # 深度信息
    def get_depth(self, symbol: str = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#38a975b802

        Name  	Type	Mandatory	Description
        symbol	str 	YES
        limit 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_depth, **to_local(locals()))

    # 近期成交列表
    def get_trades(self, symbol: str = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#2c5e424c25

        Name  	Type	Mandatory	Description
        symbol	str 	YES
        limit 	int 	NO       	默认 500; 最大值 1000.
        '''
        return self.send_request(*self.endpoints.get_trades, **to_local(locals()))

    # 查询历史成交 (MARKET_DATA)
    def get_historicalTrades(self, symbol: str = '', limit: int = '', fromId: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#market_data

        Name  	Type	Mandatory	Description
        symbol	str 	YES
        limit 	int 	NO       	默认 500; 最大值 1000.
        fromId	int 	NO       	从哪一条成交id开始返回. 缺省返回最近的成交记录。
        '''
        return self.send_request(*self.endpoints.get_historicalTrades, **to_local(locals()))

    # 近期成交(归集)
    def get_aggTrades(self, symbol: str = '', fromId: int = '', startTime: int = '', endTime: int = '',
                      limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#c59e471e81

        Name     	Type	Mandatory	Description
        symbol   	str 	YES
        fromId   	int 	NO       	从包含fromId的成交id开始返回结果
        startTime	int 	NO       	从该时刻之后的成交记录开始返回结果
        endTime  	int 	NO       	返回该时刻为止的成交记录
        limit    	int 	NO       	默认 500; 最大 1000.
        '''
        return self.send_request(*self.endpoints.get_aggTrades, **to_local(locals()))

    # K线数据
    def get_klines(self, symbol: str = '', interval: str = '', startTime: int = '', endTime: int = '', limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#k

        Name     	Type	Mandatory	Description
        symbol   	str 	YES
        interval 	str 	YES      	详见枚举定义：K线间隔
        startTime	int 	NO
        endTime  	int 	NO
        limit    	int 	NO       	默认 500; 最大 1000.
        '''
        return self.send_request(*self.endpoints.get_klines, **to_local(locals()))

    # 当前平均价格
    def get_avgPrice(self, symbol: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#3b4f48cdbb

        Name  	Type	Mandatory	Description
        symbol	str 	YES
        '''
        return self.send_request(*self.endpoints.get_avgPrice, **to_local(locals()))

    # UIK线数据
    def get_uiKlines(self, symbol: str = '', interval: str = '', startTime: int = '', endTime: int = '',
                     limit: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#uik

        Name     	Type	Mandatory	Description
        symbol   	str 	YES
        interval 	str 	YES
        startTime	int 	NO
        endTime  	int 	NO
        limit    	int 	NO       	默认 500; 最大 1000.
        '''
        return self.send_request(*self.endpoints.get_uiKlines, **to_local(locals()))

    # 24hr 价格变动情况
    def get_ticker_24hr(self, symbol: str = '', symbols: str = '', type: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#24hr

        Name   	Type	Mandatory	Description
        symbol 	str 	NO
        symbols	str 	NO
        type   	str 	NO
        '''
        return self.send_request(*self.endpoints.get_ticker_24hr, **to_local(locals()))

    # 最新价格
    def get_ticker_price(self, symbol: str = '', symbols: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#8ff46b58de

        Name   	Type	Mandatory	Description
        symbol 	str 	NO
        symbols	str 	NO
        '''
        return self.send_request(*self.endpoints.get_ticker_price, **to_local(locals()))

    # 当前最优挂单
    def get_ticker_bookTicker(self, symbol: str = '', symbols: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#5393cd07b4

        Name   	Type	Mandatory	Description
        symbol 	str 	NO
        symbols	str 	NO
        '''
        return self.send_request(*self.endpoints.get_ticker_bookTicker, **to_local(locals()))

    # 滚动窗口价格变动统计
    def get_ticker(self, symbol: str = '', symbols: list = [], windowSize: str = '', type: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#4175e32579
        Name   	    Type	Mandatory	Description
        symbol 	    str 	YES         提供 symbol或者symbols 其中之一 symbols 可以传入的格式: ["BTCUSDT","BNBUSDT"] or %5B%22BTCUSDT%22,%22BNBUSDT%22%5D
        symbols	    str
        windowSize  str     NO          默认为 1d windowSize 支持的值: 如果是分钟: 1m,2m....59m 如果是小时: 1h, 2h....23h 如果是天: 1d...7d
        type        str     NO          可接受的参数: FULL or MINI. 如果不提供, 默认值为 FULL
        '''
        return self.send_request(*self.endpoints.get_ticker, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 子母账户
class _SubAccountEndpoints():
    set_virtualSubAccount = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/virtualSubAccount', True]  # 创建虚拟子账户（适用主账户）
    get_list = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/list', True]  # 查询子账户列表（适用主账户）
    get_transfer_history = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/sub/transfer/history', True]  # 查询子账户现货资金划转历史 (适用主账户)
    get_futures_internalTransfer = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/futures/internalTransfer', True]  # 查询子账户合约资金划转历史 (适用主账户)
    set_futures_internalTransfer = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/futures/internalTransfer', True]  # 执行子账户合约资金划转 (适用主账户)
    get_assets = ['https://api.binance.com/','GET', '/sapi/v3/sub-account/assets', True]  # 查询子账户资产 (适用主账户)
    get_spotSummary = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/spotSummary', True]  # 查询子账户现货资产汇总 (适用主账户)
    get_capital_deposit_subAddress = ['https://api.binance.com/','GET', '/sapi/v1/capital/deposit/subAddress', True]  # 获取子账户充值地址 (适用主账户)
    get_capital_deposit_subHisrec = ['https://api.binance.com/','GET', '/sapi/v1/capital/deposit/subHisrec', True]  # 获取子账户充值记录 (适用主账户)
    get_status = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/status', True]  # 查询子账户Margin/Futures状态 (适用主账户)
    set_margin_enable = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/margin/enable', True]  # 为子账户开通Margin (适用主账户)
    get_margin_account = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/margin/account', True]  # 查询子账户Margin账户详情 (适用主账户)
    get_margin_accountSummary = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/margin/accountSummary', True]  # 查询子账户Margin账户汇总 (适用主账户)
    set_futures_enable = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/futures/enable', True]  # 为子账户开通Futures (适用主账户)
    set_futures_transfer = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/futures/transfer', True]  # 子账户Futures划转 (仅适用主账户)
    set_margin_transfer = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/margin/transfer', True]  # 子账户Margin划转 (仅适用主账户)
    set_transfer_subToSub = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/transfer/subToSub', True]  # 向共同主账户下的子账户主动划转 (仅适用子账户)
    set_transfer_subToMaster = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/transfer/subToMaster', True]  # 向主账户主动划转 (仅适用子账户)
    get_transfer_subUserHistory = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/transfer/subUserHistory', True]  # 查询子账户划转历史 (仅适用子账户)
    set_universalTransfer = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/universalTransfer', True]  # 子母账户万能划转 (适用主账户)
    get_universalTransfer = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/universalTransfer', True]  # 查询子母账户万能划转历史 (适用主账户)
    get_futures_account = ['https://api.binance.com/','GET', '/sapi/v2/sub-account/futures/account', True]  # 查询子账户Futures账户详情V2 (适用主账户)
    get_futures_accountSummary = ['https://api.binance.com/','GET', '/sapi/v2/sub-account/futures/accountSummary', True]  # 查询子账户Futures账户汇总V2 (适用主账户)
    get_futures_positionRisk = ['https://api.binance.com/','GET', '/sapi/v2/sub-account/futures/positionRisk', True]  # 查询子账户合约持仓信息V2 (仅适用主账户)
    set_blvt_enable = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/blvt/enable', True]  # 为子账户开通杠杆代币 (适用母账户)
    sert_subAccountApi_ipRestriction = ['','POST','/sapi/v1/sub-account/subAccountApi/ipRestriction',True]  #为子账户API Key开启/关闭IP白名单 (适用母账户)
    set_subAccountApi_ipRestriction_ipList = ['https://api.binance.com/','POST', '/sapi/v1/sub-account/subAccountApi/ipRestriction/ipList', True]  # 为子账户API Key添加IP白名单 (适用母账户)
    get_subAccountApi_ipRestriction = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/subAccountApi/ipRestriction', True]  # 查询子账户API Key IP白名单 (适用母账户)
    delete_subAccountApi_ipRestriction_ipList = ['https://api.binance.com/','DELETE', '/sapi/v1/sub-account/subAccountApi/ipRestriction/ipList', True]  # 删除子账户API Key IP白名单 (适用母账户)
    get_apiRestrictions_ipRestriction_thirdPartyList = ['https://api.binance.com/','GET', '/sapi/v1/sub-account/apiRestrictions/ipRestriction/thirdPartyList', False]  # 取得子帳戶API key IP三方名單 (适用母账户)
    set_subAccountApi_ipRestriction = ['https://api.binance.com/','POST', '/sapi/v2/sub-account/subAccountApi/ipRestriction', True]  # 为子账户API Key更新IP白名单 (适用母账户)
    set_managed_subaccount_deposit = ['https://api.binance.com/','POST', '/sapi/v1/managed-subaccount/deposit', True]  # 投资人账户为托管子账户充值资产 (适用投资人母账户)
    get_managed_subaccount_asset = ['https://api.binance.com/','GET', '/sapi/v1/managed-subaccount/asset', True]  # 投资人账户查询托管子账户资产 (适用投资人母账户)
    set_managed_subaccount_withdraw = ['https://api.binance.com/','POST', '/sapi/v1/managed-subaccount/withdraw', True]  # 投资人账户为托管子账户提币资产 (适用投资人母账户)
    get_managed_subaccount_accountSnapshot = ['https://api.binance.com/','GET', '/sapi/v1/managed-subaccount/accountSnapshot', True]  # 查询托管子账户资产快照 (适用投资人母账户)


class SubAccount(Client):
    endpoints = _SubAccountEndpoints

    # 创建虚拟子账户（适用主账户）
    def set_virtualSubAccount(self, subAccountString: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#85bc268690

        Name            	Type	Mandatory	Description
        subAccountString	str 	YES      	请输入字符串，我们将为您创建一个虚拟邮箱进行注册
        recvWindow      	int 	NO
        '''
        return self.send_request(*self.endpoints.set_virtualSubAccount, **to_local(locals()))

    # 查询子账户列表（适用主账户）
    def get_list(self, email: str = '', isFreeze: str = '', page: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#bd2019e819

        Name      	Type	Mandatory	Description
        email     	str 	NO       	Sub-account email
        isFreeze  	str 	NO       	true or false
        page      	int 	NO       	默认: 1
        limit     	int 	NO       	默认: 1, 最大: 200
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_list, **to_local(locals()))

    # 查询子账户现货资金划转历史 (适用主账户)
    def get_transfer_history(self, fromEmail: str = '', toEmail: str = '', startTime: int = '', endTime: int = '',
                             page: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#13801cd07b

        Name      	Type	Mandatory	Description
        fromEmail 	str 	NO
        toEmail   	str 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        page      	int 	NO       	默认: 1
        limit     	int 	NO       	默认: 500
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_transfer_history, **to_local(locals()))

    # 查询子账户合约资金划转历史 (适用主账户)
    def get_futures_internalTransfer(self, email: str = '', futuresType: int = '', startTime: int = '',
                                     endTime: int = '', page: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#c144d4d678

        Name       	Type	Mandatory	Description
        email      	str 	YES
        futuresType	int 	YES      	1:USDT合约，2: 币本位合约
        startTime  	int 	NO       	默认返回100天内历史记录
        endTime    	int 	NO       	默认返回100天内历史记录
        page       	int 	NO       	默认值: 1
        limit      	int 	NO       	默认值: 50, 最大值：500
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_futures_internalTransfer, **to_local(locals()))

    # 执行子账户合约资金划转 (适用主账户)
    def set_futures_internalTransfer(self, fromEmail: str = '', toEmail: str = '', futuresType: int = '',
                                     asset: str = '', amount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#b410330a48

        Name       	Type            	Mandatory	Description
        fromEmail  	str             	YES
        toEmail    	str             	YES
        futuresType	int             	YES      	1:USDT合约， 2: 币本位合约
        asset      	str             	YES
        amount     	Union[float,int]	YES
        recvWindow 	int             	NO
        '''
        return self.send_request(*self.endpoints.set_futures_internalTransfer, **to_local(locals()))

    # 查询子账户资产 (适用主账户)
    def get_assets(self, email: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#46f686d3aa

        Name      	Type	Mandatory	Description
        email     	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_assets, **to_local(locals()))

    # 查询子账户现货资产汇总 (适用主账户)
    def get_spotSummary(self, email: str = '', page: int = '', size: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#52f120c8f6

        Name      	Type	Mandatory	Description
        email     	str 	NO       	子账户邮箱
        page      	int 	NO       	分页，默认 1
        size      	int 	NO       	单页条目数, 默认 10, 最大 20
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_spotSummary, **to_local(locals()))

    # 获取子账户充值地址 (适用主账户)
    def get_capital_deposit_subAddress(self, email: str = '', coin: str = '', network: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#7c015974a6

        Name      	Type	Mandatory	Description
        email     	str 	YES
        coin      	str 	YES
        network   	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_capital_deposit_subAddress, **to_local(locals()))

    # 获取子账户充值记录 (适用主账户)
    def get_capital_deposit_subHisrec(self, email: str = '', coin: str = '', status: int = '', startTime: int = '',
                                      endTime: int = '', limit: int = '', offset: int = '', recvWindow: int = '',
                                      txId: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#0baad7eee0

        Name      	Type	Mandatory	Description
        email     	str 	YES
        coin      	str 	NO
        status    	int 	NO       	0(0:pending,6: credited but cannot withdraw, 1:success)
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO
        offset    	int 	NO       	default:0
        recvWindow	int 	NO
        txId      	str 	NO
        '''
        return self.send_request(*self.endpoints.get_capital_deposit_subHisrec, **to_local(locals()))

    # 查询子账户Margin/Futures状态 (适用主账户)
    def get_status(self, email: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin-futures

        Name      	Type	Mandatory	Description
        email     	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_status, **to_local(locals()))

    # 为子账户开通Margin (适用主账户)
    def set_margin_enable(self, email: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin

        Name      	Type	Mandatory	Description
        email     	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_margin_enable, **to_local(locals()))

    # 查询子账户Margin账户详情 (适用主账户)
    def get_margin_account(self, email: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin-2

        Name      	Type	Mandatory	Description
        email     	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_margin_account, **to_local(locals()))

    # 查询子账户Margin账户汇总 (适用主账户)
    def get_margin_accountSummary(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin-3

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_margin_accountSummary, **to_local(locals()))

    # 为子账户开通Futures (适用主账户)
    def set_futures_enable(self, email: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#futures

        Name      	Type	Mandatory	Description
        email     	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_futures_enable, **to_local(locals()))

    # 子账户Futures划转 (仅适用主账户)
    def set_futures_transfer(self, email: str = '', asset: str = '', amount: Union[float, int] = '', type: int = '',
                             recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#futures-4

        Name      	Type            	Mandatory	Description
        email     	str             	YES
        asset     	str             	YES      	划转资产, e.g., USDT
        amount    	Union[float,int]	YES      	划转数量
        type      	int             	YES      	1: 由子账户的现货账户划转至其USDT本位合约账户; 2: 由子账户的USDT本位合约账户划转至其现货账户； 3:由子账户现货账户划转至其COIN本位合约账户；4: 由子账户COIN本位合约账户划转至其现货账户
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_futures_transfer, **to_local(locals()))

    # 子账户Margin划转 (仅适用主账户)
    def set_margin_transfer(self, email: str = '', asset: str = '', amount: Union[float, int] = '', type: int = '',
                            recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#margin-4

        Name      	Type            	Mandatory	Description
        email     	str             	YES
        asset     	str             	YES      	划转资产, e.g., USDT
        amount    	Union[float,int]	YES      	划转数量
        type      	int             	YES      	1: 由子账户的现货账户划转至其杠杆账户; 2: 由子账户的杠杆账户划转至其现货账户
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_margin_transfer, **to_local(locals()))

    # 向共同主账户下的子账户主动划转 (仅适用子账户)
    def set_transfer_subToSub(self, toEmail: str = '', asset: str = '', amount: Union[float, int] = '',
                              recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#84d71c39a1

        Name      	Type            	Mandatory	Description
        toEmail   	str             	YES
        asset     	str             	YES
        amount    	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_transfer_subToSub, **to_local(locals()))

    # 向主账户主动划转 (仅适用子账户)
    def set_transfer_subToMaster(self, asset: str = '', amount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#19a57905df

        Name      	Type            	Mandatory	Description
        asset     	str             	YES
        amount    	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_transfer_subToMaster, **to_local(locals()))

    # 查询子账户划转历史 (仅适用子账户)
    def get_transfer_subUserHistory(self, asset: str = '', type: int = '', startTime: int = '', endTime: int = '',
                                    limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#f38ebd8fc2

        Name      	Type	Mandatory	Description
        asset     	str 	NO       	如不提供，返回所有asset 划转记录
        type      	int 	NO       	1: transfer in, 2: transfer out; 如不提供，返回transfer out方向划转记录
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	默认值: 500
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_transfer_subUserHistory, **to_local(locals()))

    # 子母账户万能划转 (适用主账户)
    def set_universalTransfer(self, fromEmail: str = '', toEmail: str = '', fromAccountType: str = '',
                              toAccountType: str = '', clientTranId: str = '', symbol: str = '', asset: str = '',
                              amount: Union[float, int] = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#c2d564529d

        Name           	Type            	Mandatory	Description
        fromEmail      	str             	NO
        toEmail        	str             	NO
        fromAccountType	str             	YES      	"SPOT","USDT_FUTURE","COIN_FUTURE","MARGIN"(Cross),"ISOLATED_MARGIN"
        toAccountType  	str             	YES      	"SPOT","USDT_FUTURE","COIN_FUTURE","MARGIN"(Cross),"ISOLATED_MARGIN"
        clientTranId   	str             	NO       	不可重复
        symbol         	str             	NO       	仅在ISOLATED_MARGIN类型下使用
        asset          	str             	YES
        amount         	Union[float,int]	YES
        recvWindow     	int             	NO
        '''
        return self.send_request(*self.endpoints.set_universalTransfer, **to_local(locals()))

    # 查询子母账户万能划转历史 (适用主账户)
    def get_universalTransfer(self, fromEmail: str = '', toEmail: str = '', clientTranId: str = '', startTime: int = '',
                              endTime: int = '', page: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#5f274ef2c2

        Name        	Type	Mandatory	Description
        fromEmail   	str 	NO
        toEmail     	str 	NO
        clientTranId	str 	NO
        startTime   	int 	NO
        endTime     	int 	NO
        page        	int 	NO       	默认 1
        limit       	int 	NO       	默认 500, 最大 500
        recvWindow  	int 	NO
        '''
        return self.send_request(*self.endpoints.get_universalTransfer, **to_local(locals()))

    # 查询子账户Futures账户详情V2 (适用主账户)
    def get_futures_account(self, email: str = '', futuresType: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#futures-v2

        Name       	Type	Mandatory	Description
        email      	str 	YES
        futuresType	int 	YES      	1:USDT Margined Futures, 2:COIN Margined Futures
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_futures_account, **to_local(locals()))

    # 查询子账户Futures账户汇总V2 (适用主账户)
    def get_futures_accountSummary(self, futuresType: int = '', page: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#futures-v2-2

        Name       	Type	Mandatory	Description
        futuresType	int 	YES      	1:USDT Margined Futures, 2:COIN Margined Futures
        page       	int 	NO       	default:1
        limit      	int 	NO       	default:10, max:20
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_futures_accountSummary, **to_local(locals()))

    # 查询子账户合约持仓信息V2 (仅适用主账户)
    def get_futures_positionRisk(self, email: str = '', futuresType: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#v2

        Name       	Type	Mandatory	Description
        email      	str 	YES
        futuresType	int 	YES      	1:USDT Margined Futures, 2:COIN Margined Futures
        recvWindow 	int 	NO
        '''
        return self.send_request(*self.endpoints.get_futures_positionRisk, **to_local(locals()))

    # 为子账户开通杠杆代币 (适用母账户)
    def set_blvt_enable(self, email: str = '', enableBlvt: bool = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#60e8efdca2

        Name      	Type	Mandatory	Description
        email     	str 	YES      	Sub-account email
        enableBlvt	bool	YES      	Only true for now
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_blvt_enable, **to_local(locals()))

    # 为子账户API Key开启/关闭IP白名单 (适用母账户)
    def sert_subAccountApi_ipRestriction(self, email: str = '', subAccountApiKey: str = '', ipRestrict: bool = '',
                                         recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#api-key-ip

        Name            	Type	Mandatory	Description
        email           	str 	YES      	Sub-account email
        subAccountApiKey	str 	YES
        ipRestrict      	bool	YES      	true or false
        recvWindow      	int 	NO
        '''
        return self.send_request(*self.endpoints.sert_subAccountApi_ipRestriction, **to_local(locals()))

    # 为子账户API Key添加IP白名单 (适用母账户)
    def set_subAccountApi_ipRestriction_ipList(self, email: str = '', subAccountApiKey: str = '', ipAddress: str = '',
                                               recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#api-key-ip-2

        Name            	Type	Mandatory	Description
        email           	str 	YES      	Sub-account email
        subAccountApiKey	str 	YES
        ipAddress       	str 	NO       	可批量添加，用逗号分隔。单API Key最多可添加30个
        recvWindow      	int 	NO
        '''
        return self.send_request(*self.endpoints.set_subAccountApi_ipRestriction_ipList, **to_local(locals()))

    # 查询子账户API Key IP白名单 (适用母账户)
    def get_subAccountApi_ipRestriction(self, email: str = '', subAccountApiKey: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#api-key-ip-3

        Name            	Type	Mandatory	Description
        email           	str 	YES      	Sub-account email
        subAccountApiKey	str 	YES
        recvWindow      	int 	NO
        '''
        return self.send_request(*self.endpoints.get_subAccountApi_ipRestriction, **to_local(locals()))

    # 删除子账户API Key IP白名单 (适用母账户)
    def delete_subAccountApi_ipRestriction_ipList(self, email: str = '', subAccountApiKey: str = '',
                                                  ipAddress: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#api-key-ip-4

        Name            	Type	Mandatory	Description
        email           	str 	YES      	Sub-account email
        subAccountApiKey	str 	YES
        ipAddress       	str 	NO       	可批量删除，用逗号分隔
        recvWindow      	int 	NO
        '''
        return self.send_request(*self.endpoints.delete_subAccountApi_ipRestriction_ipList, **to_local(locals()))

    # 取得子帳戶API key IP三方名單 (适用母账户)
    def get_apiRestrictions_ipRestriction_thirdPartyList(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#api-key-ip-5

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_apiRestrictions_ipRestriction_thirdPartyList, **to_local(locals()))

    # 为子账户API Key更新IP白名单 (适用母账户)
    def set_subAccountApi_ipRestriction(self, email: str = '', subAccountApiKey: str = '', status: str = '',
                                        thirdPartyName: str = '', ipAddress: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#api-key-ip-6

        Name            	Type	Mandatory	Description
        email           	str 	YES      	Sub-account email
        subAccountApiKey	str 	YES
        status          	str 	YES      	IP限制状态。1或不填入(null) = IP未受限。2 = 仅限受信任IP访问。3 = 仅限受信任三方IP访问。
        thirdPartyName  	str 	NO       	填入三方名称
        ipAddress       	str 	NO       	可批量填入IP，以逗号区隔
        recvWindow      	int 	NO
        '''
        return self.send_request(*self.endpoints.set_subAccountApi_ipRestriction, **to_local(locals()))

    # 投资人账户为托管子账户充值资产 (适用投资人母账户)
    def set_managed_subaccount_deposit(self, toEmail: str = '', asset: str = '', amount: Union[float, int] = '',
                                       recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#6b06e66554

        Name      	Type            	Mandatory	Description
        toEmail   	str             	YES
        asset     	str             	YES
        amount    	Union[float,int]	YES
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_managed_subaccount_deposit, **to_local(locals()))

    # 投资人账户查询托管子账户资产 (适用投资人母账户)
    def get_managed_subaccount_asset(self, email: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#55063e46dd

        Name      	Type	Mandatory	Description
        email     	str 	YES
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_managed_subaccount_asset, **to_local(locals()))

    # 投资人账户为托管子账户提币资产 (适用投资人母账户)
    def set_managed_subaccount_withdraw(self, fromEmail: str = '', asset: str = '', amount: Union[float, int] = '',
                                        transferDate: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#6105aa08c4

        Name        	Type            	Mandatory	Description
        fromEmail   	str             	YES
        asset       	str             	YES
        amount      	Union[float,int]	YES
        transferDate	int             	NO       	提币会自动发生在选择的日期(UTC0)，如果没有选择日期，提币会立即生效
        recvWindow  	int             	NO
        '''
        return self.send_request(*self.endpoints.set_managed_subaccount_withdraw, **to_local(locals()))

    # 查询托管子账户资产快照 (适用投资人母账户)
    def get_managed_subaccount_accountSnapshot(self, email: str = '', type: str = '', startTime: int = '',
                                               endTime: int = '', limit: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#2c2c5f3fa7

        Name      	Type	Mandatory	Description
        email     	str 	YES
        type      	str 	YES      	"SPOT"（现货）, "MARGIN"（全仓）, "FUTURES"（U本位合约）
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	min 7, max 30, default 7
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_managed_subaccount_accountSnapshot, **to_local(locals()))



from paux.param import to_local
from pbinance.client import Client
from typing import Union


# 钱包接口
class _WalletEndpoints():
    get_system_status = ['https://api.binance.com/', 'GET', '/sapi/v1/system/status', False]  # 系统状态(System)
    get_capital_config_getall = ['https://api.binance.com/', 'GET', '/sapi/v1/capital/config/getall', True]  # 获取所有币信息 (USER_DATA)
    get_accountSnapshot = ['https://api.binance.com/', 'GET', '/sapi/v1/accountSnapshot', True]  # 查询每日资产快照 (USER_DATA)
    set_account_disableFastWithdrawSwitch = ['https://api.binance.com/', 'POST', '/sapi/v1/account/disableFastWithdrawSwitch', True]  # 关闭站内划转 (USER_DATA)
    set_account_enableFastWithdrawSwitch = ['https://api.binance.com/', 'POST', '/sapi/v1/account/enableFastWithdrawSwitch', True]  # 开启站内划转 (USER_DATA)
    set_capital_withdraw_apply = ['https://api.binance.com/', 'POST', '/sapi/v1/capital/withdraw/apply', True]  # 提币 (USER_DATA)
    get_capital_deposit_hisrec = ['https://api.binance.com/', 'GET', '/sapi/v1/capital/deposit/hisrec', True]  # 获取充值历史(支持多网络) (USER_DATA)
    get_capital_withdraw_history = ['https://api.binance.com/', 'GET', '/sapi/v1/capital/withdraw/history', True]  # 获取提币历史 (支持多网络) (USER_DATA)
    get_capital_deposit_address = ['https://api.binance.com/', 'GET', '/sapi/v1/capital/deposit/address', True]  # 获取充值地址 (支持多网络) (USER_DATA)
    get_account_status = ['https://api.binance.com/', 'GET', '/sapi/v1/account/status', False]  # 账户状态 (USER_DATA)
    get_account_apiTradingStatus = ['https://api.binance.com/', 'GET', '/sapi/v1/account/apiTradingStatus', True]  # 账户API交易状态(USER_DATA)
    get_asset_dribblet = ['https://api.binance.com/', 'GET', '/sapi/v1/asset/dribblet', True]  # 小额资产转换BNB历史 (USER_DATA)
    set_asset_dust_btc = ['https://api.binance.com/', 'POST', '/sapi/v1/asset/dust-btc', True]  # 获取可以转换成BNB的小额资产 (USER_DATA)
    set_asset_dust = ['https://api.binance.com/', 'POST', '/sapi/v1/asset/dust', True]  # 小额资产转换 (USER_DATA)
    get_asset_assetDividend = ['https://api.binance.com/', 'GET', '/sapi/v1/asset/assetDividend', True]  # 资产利息记录 (USER_DATA)
    get_asset_assetDetail = ['https://api.binance.com/', 'GET', '/sapi/v1/asset/assetDetail', True]  # 上架资产详情 (USER_DATA)
    get_asset_tradeFee = ['https://api.binance.com/', 'GET', '/sapi/v1/asset/tradeFee', True]  # 交易手续费率查询 (USER_DATA)
    set_set_asset_transfer = ['https://api.binance.com/', 'POST', '/sapi/v1/asset/transfer', True]  # 用户万向划转 (USER_DATA)
    get_get_asset_transfer = ['https://api.binance.com/', 'GET', '/sapi/v1/asset/transfer', True]  # 查询用户万向划转历史 (USER_DATA)
    set_asset_get_funding_asset = ['https://api.binance.com/', 'POST', '/sapi/v1/asset/get-funding-asset', True]  # 资金账户 (USER_DATA)
    set_asset_getUserAsset = ['https://api.binance.com/', 'POST', '/sapi/v3/asset/getUserAsset', False]  # 用户持仓 (USER_DATA)
    set_asset_convert_transfer = ['https://api.binance.com/', 'POST', '/sapi/v1/asset/convert-transfer', False]  # 稳定币自动兑换划转 (TRADE)
    get_asset_convert_transfer_queryByPage = ['https://api.binance.com/', 'GET', '/sapi/v1/asset/convert-transfer/queryByPage', False]  # # 稳定币自动兑换划转查询 (USER_DATA)
    get_asset_ledger_transfer_cloud_mining_queryByPage = ['https://api.binance.com/', 'GET', '/sapi/v1/asset/ledger-transfer/cloud-mining/queryByPage', False]  # 云算力历史记录分页查询 (USER_DATA)
    get_account_apiRestrictions = ['https://api.binance.com/', 'GET', '/sapi/v1/account/apiRestrictions', True]  # 查询用户API Key权限 (USER_DATA)

class Wallet(Client):
    endpoints = _WalletEndpoints

    # 系统状态(System)
    def get_system_status(self):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#system
        '''
        return self.send_request(*self.endpoints.get_system_status, **to_local(locals()))

    # 获取所有币信息 (USER_DATA)
    def get_capital_config_getall(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_capital_config_getall, **to_local(locals()))

    # 查询每日资产快照 (USER_DATA)
    def get_accountSnapshot(self, type: str = '', startTime: int = '', endTime: int = '', limit: int = '',
                            recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-2

        Name      	Type	Mandatory	Description
        type      	str 	YES      	"SPOT", "MARGIN", "FUTURES"
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	min 7, max 30, default 7
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_accountSnapshot, **to_local(locals()))

    # 关闭站内划转 (USER_DATA)
    def set_account_disableFastWithdrawSwitch(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-3

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_account_disableFastWithdrawSwitch, **to_local(locals()))

    # 开启站内划转 (USER_DATA)
    def set_account_enableFastWithdrawSwitch(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-4

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_account_enableFastWithdrawSwitch, **to_local(locals()))

    # 提币 (USER_DATA)
    def set_capital_withdraw_apply(self, coin: str = '', withdrawOrderId: str = '', network: str = '',
                                   address: str = '', addressTag: str = '', amount: Union[float, int] = '',
                                   transactionFeeFlag: bool = '', name: str = '', walletType: int = '',
                                   recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-5

        Name              	Type            	Mandatory	Description
        coin              	str             	YES
        withdrawOrderId   	str             	NO       	自定义提币ID
        network           	str             	NO       	提币网络
        address           	str             	YES      	提币地址
        addressTag        	str             	NO       	某些币种例如 XRP,XMR 允许填写次级地址标签
        amount            	Union[float,int]	YES      	数量
        transactionFeeFlag	bool            	NO
        name              	str             	NO
        walletType        	int             	NO       	表示出金使用的钱包，0为现货钱包，1为资金钱包。默认walletType为"充币账户"是您设置在钱包->现货账户或资金账户->充值。
        recvWindow        	int             	NO
        '''
        return self.send_request(*self.endpoints.set_capital_withdraw_apply, **to_local(locals()))

    # 获取充值历史(支持多网络) (USER_DATA)
    def get_capital_deposit_hisrec(self, coin: str = '', status: int = '', startTime: int = '', endTime: int = '',
                                   offset: int = '', limit: int = '', recvWindow: int = '', txId: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-6

        Name      	Type	Mandatory	Description
        coin      	str 	NO
        status    	int 	NO       	0(0:pending,6: credited but cannot withdraw, 1:success)
        startTime 	int 	NO       	默认当前时间90天前的时间戳
        endTime   	int 	NO       	默认当前时间戳
        offset    	int 	NO       	默认:0
        limit     	int 	NO       	默认：1000，最大1000
        recvWindow	int 	NO
        txId      	str 	NO
        '''
        return self.send_request(*self.endpoints.get_capital_deposit_hisrec, **to_local(locals()))

    # 获取提币历史 (支持多网络)  (USER_DATA)
    def get_capital_withdraw_history(self, coin: str = '', withdrawOrderId: str = '', status: int = '',
                                     offset: int = '', limit: int = '', startTime: int = '', endTime: int = '',
                                     recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-7

        Name           	Type	Mandatory	Description
        coin           	str 	NO
        withdrawOrderId	str 	NO
        status         	int 	NO       	0(0:已发送确认Email,1:已被用户取消 2:等待确认 3:被拒绝 4:处理中 5:提现交易失败 6 提现完成)
        offset         	int 	NO
        limit          	int 	NO       	默认：1000， 最大：1000
        startTime      	int 	NO       	默认当前时间90天前的时间戳
        endTime        	int 	NO       	默认当前时间戳
        recvWindow     	int 	NO
        '''
        return self.send_request(*self.endpoints.get_capital_withdraw_history, **to_local(locals()))

    # 获取充值地址 (支持多网络) (USER_DATA)
    def get_capital_deposit_address(self, coin: str = '', network: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-8

        Name      	Type	Mandatory	Description
        coin      	str 	YES
        network   	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_capital_deposit_address, **to_local(locals()))

    # 账户状态 (USER_DATA)
    def get_account_status(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-9

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_account_status, **to_local(locals()))

    # 账户API交易状态(USER_DATA)
    def get_account_apiTradingStatus(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#api-user_data

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_account_apiTradingStatus, **to_local(locals()))

    # 小额资产转换BNB历史 (USER_DATA)
    def get_asset_dribblet(self, startTime: int = '', endTime: int = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#bnb-user_data

        Name      	Type	Mandatory	Description
        startTime 	int 	NO
        endTime   	int 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_asset_dribblet, **to_local(locals()))

    # 获取可以转换成BNB的小额资产 (USER_DATA)
    def set_asset_dust_btc(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#bnb-user_data-2

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_asset_dust_btc, **to_local(locals()))

    # 小额资产转换 (USER_DATA)
    def set_asset_dust(self, asset: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-10

        Name      	Type	Mandatory	Description
        asset     	str 	YES      	正在转换的资产。 例如：asset = BTC＆asset = USDT
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.set_asset_dust, **to_local(locals()))

    # 资产利息记录 (USER_DATA)
    def get_asset_assetDividend(self, asset: str = '', startTime: int = '', endTime: int = '', limit: int = '',
                                recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-11

        Name      	Type	Mandatory	Description
        asset     	str 	NO
        startTime 	int 	NO
        endTime   	int 	NO
        limit     	int 	NO       	Default 20, max 500
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_asset_assetDividend, **to_local(locals()))

    # 上架资产详情 (USER_DATA)
    def get_asset_assetDetail(self, asset: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-12

        Name      	Type	Mandatory	Description
        asset     	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_asset_assetDetail, **to_local(locals()))

    # 交易手续费率查询 (USER_DATA)
    def get_asset_tradeFee(self, symbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-13

        Name      	Type	Mandatory	Description
        symbol    	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_asset_tradeFee, **to_local(locals()))

    # 用户万向划转 (USER_DATA)
    def set_set_asset_transfer(self, type: str = '', asset: str = '', amount: Union[float, int] = '',
                               fromSymbol: str = '', toSymbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-14

        Name      	Type            	Mandatory	Description
        type      	str             	YES
        asset     	str             	YES
        amount    	Union[float,int]	YES
        fromSymbol	str             	NO
        toSymbol  	str             	NO
        recvWindow	int             	NO
        '''
        return self.send_request(*self.endpoints.set_set_asset_transfer, **to_local(locals()))

    # 查询用户万向划转历史 (USER_DATA)
    def get_get_asset_transfer(self, type: str = '', startTime: int = '', endTime: int = '', current: int = '',
                               size: int = '', fromSymbol: str = '', toSymbol: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-15

        Name      	Type	Mandatory	Description
        type      	str 	YES
        startTime 	int 	NO
        endTime   	int 	NO
        current   	int 	NO       	默认 1
        size      	int 	NO       	默认 10, 最大 100
        fromSymbol	str 	NO
        toSymbol  	str 	NO
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_get_asset_transfer, **to_local(locals()))

    # 资金账户 (USER_DATA)
    def set_asset_get_funding_asset(self, asset: str = '', needBtcValuation: str = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-16

        Name            	Type	Mandatory	Description
        asset           	str 	NO
        needBtcValuation	str 	NO       	true or false
        recvWindow      	int 	NO
        '''
        return self.send_request(*self.endpoints.set_asset_get_funding_asset, **to_local(locals()))

    # 用户持仓 (USER_DATA)
    def set_asset_getUserAsset(self, asset: str = '', needBtcValuation: bool = '', recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-17

        Name            	Type	Mandatory	Description
        asset           	str 	NO       	如果资产为空，则查询用户所有的正资产。
        needBtcValuation	bool	NO       	是否需要返回兑换成BTC的估值
        recvWindow      	int 	NO
        '''
        return self.send_request(*self.endpoints.set_asset_getUserAsset, **to_local(locals()))

    # 稳定币自动兑换划转 (TRADE)
    def set_asset_convert_transfer(self, clientTranId: str = '', asset: str = '', amount: Union[float, int] = '',
                                   targetAsset: str = '', accountType: str = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#trade

        Name        	Type            	Mandatory	Description
        clientTranId	str             	YES      	用户自定义流水号，唯一标志，限制最短长度为20
        asset       	str             	YES      	当前资产
        amount      	Union[float,int]	YES      	数量必须为正数
        targetAsset 	str             	YES      	目标资产
        accountType 	str             	NO       	仅支持MAIN和CARD，如果为空，默认查询主账户MAIN
        '''
        return self.send_request(*self.endpoints.set_asset_convert_transfer, **to_local(locals()))

    # 稳定币自动兑换划转查询 (USER_DATA)
    def get_asset_convert_transfer_queryByPage(self, tranId: int = '', clientTranId: str = '', asset: str = '',
                                               startTime: int = '', endTime: int = '', accountType: str = '',
                                               current: int = '', size: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-18

        Name        	Type	Mandatory	Description
        tranId      	int 	NO       	流水号
        clientTranId	str 	NO       	用户自定义流水号
        asset       	str 	NO       	不传或者空字符串查全部, 匹配扣除币种和目标币种
        startTime   	int 	YES      	开始时间（包含），单位：毫秒
        endTime     	int 	YES      	结束时间（不包含），单位：毫秒
        accountType 	str 	NO       	账户类型: MAIN-主账户。CARD-资金账户。如果传入则仅返回对应wallet的记录，不传或者传null则返回该用户spot和card钱包的记录。
        current     	int 	NO       	当前页面，默认1，最小值为1
        size        	int 	NO       	页面大小，默认10，最大值为100
        '''
        return self.send_request(*self.endpoints.get_asset_convert_transfer_queryByPage, **to_local(locals()))

    # 云算力历史记录分页查询 (USER_DATA)
    def get_asset_ledger_transfer_cloud_mining_queryByPage(self, tranId: int = '', clientTranId: str = '',
                                                           asset: str = '', startTime: int = '', endTime: int = '',
                                                           current: int = '', size: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#user_data-19

        Name        	Type	Mandatory	Description
        tranId      	int 	NO       	流水号
        clientTranId	str 	NO       	外部唯一流水号
        asset       	str 	NO       	不传或者空字符串查全部
        startTime   	int 	YES      	开始时间（包含），单位：毫秒
        endTime     	int 	YES      	结束时间（不包含），单位：毫秒
        current     	int 	NO       	当前页面，默认1，最小值为1
        size        	int 	NO       	页面大小，默认10，最大值为100
        '''
        return self.send_request(*self.endpoints.get_asset_ledger_transfer_cloud_mining_queryByPage, **to_local(locals()))

    # 查询用户API Key权限 (USER_DATA)
    def get_account_apiRestrictions(self, recvWindow: int = ''):
        '''
        https://binance-docs.github.io/apidocs/spot/cn/#api-key-user_data

        Name      	Type	Mandatory	Description
        recvWindow	int 	NO
        '''
        return self.send_request(*self.endpoints.get_account_apiRestrictions, **to_local(locals()))



from pbinance import Binance, EO
from pprint import pprint

if __name__ == '__main__':
    # 实例化EO模块
    eo = EO(key='****', secret='****')  # 等同于：eo = Binance( key='****', secret='****').eo
    # 获取交易规则和交易对
    result = eo.market.get_exchangeInfo()
    pprint(result)
